@@ -90,7 +90,8 @@ namespace Lum {
     void CalcSize();
     void Layout();
     void PrepareForBackground(OS::DrawInfo* draw);
-    void Draw(int x, int y, size_t w, size_t h);
+    void Draw(OS::DrawInfo* draw,
+              int x, int y, size_t w, size_t h);
     void Hide();
 
     bool HandleMouseEvent(const OS::MouseEvent& event);
