@@ -138,7 +138,7 @@ namespace Lum {
   class LUMAPI IndexCombo : public Combo
   {
   private:
-    Model::SizeTRef model; /** The model for the current value */
+    Model::NumberRef model; /** The model for the current value */
 
   protected:
     void SetModelFromSelection();
@@ -150,7 +150,7 @@ namespace Lum {
 
     bool SetModel(Base::Model* model);
 
-    static IndexCombo* Create(Lum::Model::SizeT* index, Lum::Model::Table* table,
+    static IndexCombo* Create(Lum::Model::Number* index, Lum::Model::Table* table,
                               bool horizontalFlex=false, bool verticalFlex=false);
   };
 
