@@ -50,15 +50,15 @@ namespace Lum {
 
     private:
       Desc              description;
-      Model::SizeTRef   value;
+      Model::NumberRef  value;
       std::list<Choice> choices;
 
     public:
-      OneOfN(const Desc& description, Model::SizeT *value);
+      OneOfN(const Desc& description, Model::Number* value);
 
       OneOfN& AddChoice(const Choice& choice);
 
-      Model::SizeT* GetValue() const;
+      Model::Number* GetValue() const;
 
       const Desc& GetDesc() const;
       const std::list<Choice>& GetChoices() const;
