@@ -59,7 +59,8 @@ namespace Lum {
     void CalcSize();
     bool HandleMouseEvent(const OS::MouseEvent& event);
     void Layout();
-    void Draw(int x, int y, size_t w, size_t h);
+    void PreDrawChilds(OS::DrawInfo* draw);
+    void PostDrawChilds(OS::DrawInfo* draw);
 
     void Resync(Base::Model* model, const Base::ResyncMsg& msg);
   };
