@@ -20,9 +20,7 @@
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 */
 
-#include <list>
-
-#include <Lum/Base/Object.h>
+#include <Lum/Base/Reference.h>
 #include <Lum/Base/Singleton.h>
 
 #include <Lum/OS/DrawInfo.h>
@@ -47,7 +45,7 @@ namespace Lum {
       independend data together with some OS dependend
       handles.
     */
-    class LUMAPI Image : public ::Lum::Base::Referencable
+    class LUMAPI Image : public Base::Referencable
     {
     protected:
       bool   alpha;   /** TRUE, if there is alpha channel information in the image */
@@ -124,7 +122,7 @@ namespace Lum {
     public:
       static void Set(Factory* factory);
 
-      Factory(::Lum::OS::Display* display);
+      Factory(OS::Display* display);
       virtual ~Factory();
 
       virtual void Deinit(OS::Display* display) = 0;
