@@ -33,8 +33,8 @@ namespace Lum {
       Calendar() : Simple<Base::Calendar>() {}
       Calendar(const Base::Calendar& value) : Simple<Base::Calendar>(value) {}
 
-      void SetToCurrent();
-      void SetDate(int dayOfMonth, int month, int year);
+      bool SetToCurrent();
+      bool SetDate(int dayOfMonth, int month, int year);
     };
 
     typedef Base::Reference<Calendar> CalendarRef;
