@@ -41,11 +41,13 @@ namespace Lum {
         ColorMode                        colorMode;
         size_t                           colorDepth;
 
-        size_t                           scrWidth;
-        size_t                           scrHeight;
+        unsigned long                    screenWidth;
+        unsigned long                    screenHeight;
 
-        size_t                           workAreaWidth;
-        size_t                           workAreaHeight;
+        unsigned long                    workAreaWidth;
+        unsigned long                    workAreaHeight;
+
+        Model::ActionRef                 screenRotatedAction;
 
         unsigned long                    multiClickTime;
 
@@ -81,6 +83,8 @@ namespace Lum {
 
         void EvaluateDisplaySize();
 
+        void TriggerScreenRotated();
+
         virtual FontRef GetFontInternal(FontType type, size_t size) const;
 
         bool GetFontFromCache(FontType type, size_t size, FontRef& font) const;
@@ -91,6 +95,9 @@ namespace Lum {
         // Getter
         Type GetType() const;
         Size GetSize() const;
+        bool IsLandscape() const;
+        bool IsPortrait() const;
+        Model::Action* GetScreenRotatedAction() const;
 
         void SetAppName(const std::wstring& name);
         std::wstring GetAppName() const;
@@ -105,11 +112,11 @@ namespace Lum {
         ColorMode GetColorMode() const;
         size_t GetColorDepth() const;
 
-        size_t GetScreenWidth() const;
-        size_t GetScreenHeight() const;
+        unsigned long GetScreenWidth() const;
+        unsigned long GetScreenHeight() const;
 
-        size_t GetWorkAreaWidth() const;
-        size_t GetWorkAreaHeight() const;
+        unsigned long GetWorkAreaWidth() const;
+        unsigned long GetWorkAreaHeight() const;
 
         unsigned long GetMultiClickTime() const;
 
