@@ -117,6 +117,10 @@ namespace Lum {
 
         size_t GetSpaceHorizontal(Display::Space space) const;
         size_t GetSpaceVertical(Display::Space space) const;
+        
+        size_t GetIconWidth() const;
+        size_t GetIconHeight() const;
+        
         size_t GetMinimalButtonWidth() const;
         size_t GetFirstTabOffset() const;
         size_t GetLastTabOffset() const;
