@@ -54,12 +54,17 @@ namespace Lum {
         Model::ActionRef     menuAction;       //! Triggered if the windo should show its window (embedded app context)
 
         std::wstring         title;            //! Title of window
+
         bool                 isInited;         //! true, if the PreInit action has been triggered
         bool                 open;             //! Window is open
         bool                 inEventLoop;      //! True, if window is in event loop
         bool                 shown;            //! Window has been redrawn after open
         bool                 focus;            //! Windows has focus
-        bool                 activateOnOpen;   //! Actrivate windows afte rit opens
+
+        bool                 activateOnOpen;   //! Activate windows after it opens
+        bool                 hasMenuHint;      //! Optional hint that the window has a menu
+        ScreenOrientationHint screenOrientationHint;
+
         int                  x,y;
         size_t               width,height;
         size_t               minWidth,minHeight;
@@ -98,14 +103,21 @@ namespace Lum {
         void SetType(OS::Window::Type type);
         OS::Window::Type GetType() const;
 
+        void SetHasMenuHint(bool hasMenu);
+        bool GetHasMenuHint() const;
+
+        void SetScreenOrientationHint(ScreenOrientationHint hint);
+        ScreenOrientationHint GetScreenOrientationHint() const;
+
         void SetTitle(const std::wstring& name);
         std::wstring GetTitle() const;
 
         void SetSize(size_t width, size_t height);
-        void SetMinWidth(size_t width);
-        void SetMinHeight(size_t height);
-        void SetMaxWidth(size_t width);
-        void SetMaxHeight(size_t height);
+        void SetMinMaxSize(size_t minWidth,
+                           size_t minHeight,
+                           size_t maxWidth,
+                           size_t maxHeight);
+
         void SetPos(int x, int y);
         void SetPosition(OS::Window::Position horiz, OS::Window::Position vert);
 
