@@ -254,25 +254,36 @@ namespace Lum {
       /**
         Return the screen width in pixel.
       */
-      virtual size_t GetScreenWidth() const = 0;
+      virtual unsigned long GetScreenWidth() const = 0;
 
       /**
         Return the screen height in pixel.
       */
-      virtual size_t GetScreenHeight() const = 0;
+      virtual unsigned long GetScreenHeight() const = 0;
 
       /**
         Return the work area width in pixel. Work area is the part
         of the desktop where you can place application window on.
       */
-      virtual size_t GetWorkAreaWidth() const = 0;
+      virtual unsigned long GetWorkAreaWidth() const = 0;
 
       /**
         Return the work area height in pixel. Work area is the part
         of the desktop where you can place application window on.
       */
-      virtual size_t GetWorkAreaHeight() const = 0;
+      virtual unsigned long GetWorkAreaHeight() const = 0;
+
+      /**
+        Returns a simple hint about the estimated size of the display.
+        This hint can be used to use different GUI elements for different
+        types of screen resolutions.
+        */
       virtual Size GetSize() const = 0;
+
+      virtual bool IsLandscape() const = 0;
+      virtual bool IsPortrait() const = 0;
+
+      virtual Model::Action* GetScreenRotatedAction() const = 0;
       //@}
 
       /**
