@@ -60,6 +60,13 @@ namespace Lum {
         positionOS
       };
 
+      enum ScreenOrientationHint {
+        screenOrientationAgnostic,
+        screenOrientationLandscapeSupported,
+        screenOrientationPortraitSupported,
+        screenOrientationBothSupported
+      };
+
       class LUMAPI DrawMsg : public ::Lum::Base::ResyncMsg
       {
       public:
@@ -96,24 +103,74 @@ namespace Lum {
       //@}
 
       /**
-        @name Parent window
-      */
+        @name Window hints.
+       */
       //@{
+      /**
+        If set to true, this signals that the window contains a main menu.
+        This might have influence on the window visualization depending on the platform.
+       */
+      virtual void SetHasMenuHint(bool hasMenu) = 0;
+      /**
+        Returns true, if a menu hint has been set, else false.
+       */
+      virtual bool GetHasMenuHint() const = 0;
+
+      /**
+        If screenOrientationLandscapeSupported, the window layout is feasible under landscape
+        displays (and not under portrait), if screenOrrientationPortraitSupported i set, the window
+        layout is feasible under portrait (and not under landscape). If screenOrientationAgnostic
+        is set, this means that the default orientation of the device is supported (somewhat) but the
+        application is agnostic of screen rotation. It would snot change its layout if screen
+        orientation is changed. ScreenOrientationBothSupported means that both orientations
+        are supported and layout changes to fit to the new orientation.
+
+        If screen rotation is detected and the new orientation is supported, Illumination
+        might trigger a window reinit to initiate a layout reconstruction, expecting the window
+        to supply a layout that fits the new orientation.
+       */
+      virtual void SetScreenOrientationHint(ScreenOrientationHint hint) = 0;
+
+      /**
+        Returns true, if the landscape orientation is supported, else false.
+       */
+      virtual ScreenOrientationHint GetScreenOrientationHint() const = 0;
+      //@}
+
+      /**
+        @name Parent window
+       */
+      //@{
+      /**
+        Set the parent window of this window. A window can have a parent window. If a parent windows
+        has been set you cannot reset the parent window while the window is open. Settings
+        a parent windows doe snot influence the modality of the window, but else the visualization
+        or behaviour may be influenced.
+       */
       virtual void SetParent(Window* parent) = 0;
+      /**
+        Return the parent window or NULL.
+       */
       virtual Window* GetParent() const = 0;
       //@}
 
       /**
         @name Window title
-      */
+       */
       //@{
+      /**
+        Set the window title.
+       */
       virtual void SetTitle(const std::wstring& name) = 0;
+      /**
+        Return the current window title.
+       */
       virtual std::wstring GetTitle() const = 0;
       //@}
 
       /**
         @name Window opacity
-      */
+       */
       //@{
       virtual bool SetOpacity(double opacity) = 0;
       virtual double GetOpacity() const = 0;
@@ -125,10 +182,10 @@ namespace Lum {
       */
       //@{
       virtual void SetSize(size_t width, size_t height) = 0;
-      virtual void SetMinWidth(size_t width) = 0;
-      virtual void SetMinHeight(size_t height) = 0;
-      virtual void SetMaxWidth(size_t width) = 0;
-      virtual void SetMaxHeight(size_t height) = 0;
+      virtual void SetMinMaxSize(size_t minWidth,
+                                 size_t minHeight,
+                                 size_t maxWidth,
+                                 size_t maxHeight) = 0;
       virtual void SetPos(int x, int y) = 0;
       virtual void SetPosition(Window::Position horiz, Window::Position vert) = 0;
 
