@@ -182,6 +182,15 @@ namespace Lum {
         DBusConnection            *dbusSystem;
 #endif
 
+#if defined(HAVE_LIB_XRANDR)
+        bool                      xrandrActive;
+        int                       xrandrMinor,xrandrMajor;
+        int                       xrandrEventBase,xrandrErrorBase;
+#endif
+
+      private:
+        void GetWorkAreaDimension();
+
       public:
         Display();
         ~Display();
