@@ -43,14 +43,14 @@ namespace Lum {
     namespace X11 {
 
       class Display;
-      
+
       struct Rectangle
       {
         int    x;
         int    y;
         size_t width;
         size_t height;
-      }; 
+      };
 
       class LUMAPI Window : public OS::Base::Window
       {
@@ -104,6 +104,12 @@ namespace Lum {
 
         void Resize(size_t width, size_t height);
 
+        void SetSize(size_t width, size_t height);
+        void SetMinMaxSize(size_t minWidth,
+                           size_t minHeight,
+                           size_t maxWidth,
+                           size_t maxHeight);
+
         bool Open(bool activate);
         void Close();
         bool Show(bool activate);
