@@ -154,8 +154,11 @@ namespace Lum {
       }
     };
 
-  protected:
+  private:
     Object              *parent;        //! The parent object, may be NULL for top level objects.
+    mutable OS::Window* window;        //! The OS::Window this object belongs to.
+
+  protected:
 
     /**
       @name Internal position and dimension.
@@ -236,7 +239,6 @@ namespace Lum {
     bool                inited;        //! Has been initialized
     bool                shortcutAlways;
 
-    mutable OS::Window* window;        //! The OS::Window this object belongs to.
     OS::Window*         menu;          //! The window for a popup menu. \see Object::SetMenu
     OS::Window*         help;
 
@@ -390,7 +392,6 @@ namespace Lum {
     */
     //@{
     virtual void SetParent(Object* parent);
-    virtual Object* GetParent() const;
     //@}
 
     /**
@@ -435,6 +436,13 @@ namespace Lum {
     virtual void RecatchedFocus();
     virtual void LostFocus();
 
+    virtual Object* GetMouseGrabFocusObject() const;
+    //@}
+
+    /**
+      @name Labeling methods
+    */
+    //@{
     virtual void SetLabelObject(Object* label);
 
     /**
@@ -476,9 +484,14 @@ namespace Lum {
     virtual void ResizeHeight(size_t height);
     //@}
 
+    /**
+      @name Layout related methods
+    */
+    //@{
     virtual void SetRelayout();
     virtual bool IsLayouted() const;
     virtual void Layout();
+    //@}
 
     /**
       @name Object point inclusion methods
@@ -488,7 +501,6 @@ namespace Lum {
     virtual bool Intersect(int &x, int &y, size_t &width, size_t &height) const;
     virtual bool PointIsIn(int x, int y) const;
     virtual bool PointIsIn(const OS::MouseEvent& event) const;
-    virtual bool MouseIsIn() const;
     //@}
 
     /**
