@@ -58,7 +58,8 @@ namespace Lum {
     void SetValue(Value value);
 
     void CalcSize();
-    void Draw(int x, int y, size_t w, size_t h);
+    void Draw(OS::DrawInfo* draw,
+              int x, int y, size_t w, size_t h);
   };
 }
 
