@@ -34,18 +34,18 @@ namespace Lum {
   class LUMAPI Slider : public Control
   {
   protected:
-    OS::FrameRef   boxFrame;
-    OS::ImageRef   knob;
-    Scale          *scale;
-    long           offset;
-    long           corr;
-    Model::IntRef  model;
-    bool           selected;
-    bool           useScale;
-    int            knobX;
-    int            knobY;
-    size_t         knobWidth;
-    size_t         knobHeight;
+    OS::FrameRef     boxFrame;
+    OS::ImageRef     knob;
+    Scale            *scale;
+    long             offset;
+    long             corr;
+    Model::NumberRef model;
+    bool             selected;
+    bool             useScale;
+    int              knobX;
+    int              knobY;
+    size_t           knobWidth;
+    size_t           knobHeight;
 
   protected:
     virtual void DrawKnob(OS::DrawInfo* draw) = 0;
