@@ -35,7 +35,7 @@ namespace Lum {
   class LUMAPI Tab : public Control
   {
   private:
-    Model::SizeTRef  model;       //! numeric model defining the currently visible entry.
+    Model::NumberRef model;       //! numeric model defining the currently visible entry.
     size_t           mw;          //! width of single tab rider
     size_t           mh;          //! height of single tab rider
     Object           *lastDrawn;  //! At that time visible object during last draw
