@@ -189,6 +189,10 @@ namespace Lum {
                                                                    Catalog::typeApplication));
           }
         }
+        else {
+          std::cerr << "Cannot retrieve application data directory!" << std::endl;
+        }
+
         appCatalogSearched=true;
       }
 
@@ -273,7 +277,9 @@ namespace Lum {
       }
     }
 
-    Catalog* Catalog::LoadCatalog(const std::wstring& locale, const std::wstring& path, Type type)
+    Catalog* Catalog::LoadCatalog(const std::wstring& locale,
+                                  const std::wstring& path,
+                                  Type type)
     {
       std::vector<Path>         paths;
       std::vector<std::wstring> localeNames;
@@ -365,15 +371,16 @@ namespace Lum {
           catalogPath=paths[p];
           catalogPath.SetBaseName(name);
 
-          // For debugging 
-	  /*
+          // For debugging
+          /*
           if (type==typeApplication) {
             std::cout << "Application catalog: ";
           }
           else {
             std::cout << "Library catalog: ";
           }
-          std::cout << Lum::Base::WStringToString(catalogPath.GetPath()) << std::endl;*/
+          std::cout << Lum::Base::WStringToString(catalogPath.GetPath()) << std::endl;
+          */
 
           if (catalogPath.Exists()) {
             xml=Config::LoadConfigFromXMLFile(catalogPath.GetPath(),errorList);
@@ -408,6 +415,9 @@ namespace Lum {
             }
           }
         }
+        else {
+          std::cerr << "Catalog file '" << Lum::Base::WStringToString(catalogPath.GetPath()) << "' is not valid!" << std::endl;
+        }
 
         delete xml;
       }
