@@ -616,7 +616,7 @@ namespace Lum {
 
   bool IndexCombo::SetModel(Base::Model* model)
   {
-    this->model=dynamic_cast<Model::SizeT*>(model);
+    this->model=dynamic_cast<Model::Number*>(model);
 
     Combo::SetModel(this->model);
 
@@ -649,22 +649,22 @@ namespace Lum {
       return;
     }
 
-    Model::SizeT *m=dynamic_cast<Model::SizeT*>(model.Get());
+    Model::Number *m=dynamic_cast<Model::Number*>(model.Get());
 
     if (m==NULL) {
       selection->Clear();
       return;
     }
 
-    if (m->IsNull() || m->Get()==0 || m->Get()>tableModel->GetRows()) {
+    if (m->IsNull() || m->GetUnsignedLong()==0 || m->GetUnsignedLong()>tableModel->GetRows()) {
       selection->Clear();
     }
     else {
-      selection->SelectLine(m->Get());
+      selection->SelectLine(m->GetUnsignedLong());
     }
   }
 
-  IndexCombo* IndexCombo::Create(Lum::Model::SizeT* index,
+  IndexCombo* IndexCombo::Create(Lum::Model::Number* index,
                                  Lum::Model::Table* table,
                                  bool horizontalFlex, bool verticalFlex)
   {
