@@ -30,6 +30,18 @@ namespace Lum {
       assert(action!=NULL);
     }
 
+    Action& Action::SetImage(const std::wstring& imageFilename)
+    {
+      this->imageFilename=imageFilename;
+      
+      return *this;
+    }
+
+    std::wstring Action::GetImageFilename() const
+    {
+      return imageFilename;
+    }
+
     Action& Action::SetType(Type type)
     {
       this->type=type;
