@@ -32,7 +32,7 @@ namespace Lum {
       return description;
     }
 
-    OneOfN::OneOfN(const Desc& description, Model::SizeT *value)
+    OneOfN::OneOfN(const Desc& description, Model::Number* value)
      : description(description),
        value(value)
     {
@@ -46,7 +46,7 @@ namespace Lum {
       return *this;
     }
 
-    Model::SizeT* OneOfN::GetValue() const
+    Model::Number* OneOfN::GetValue() const
     {
       return value;
     }
