@@ -27,8 +27,11 @@ namespace Lum {
   namespace Dlg {
 
     Help::Help(const std::wstring& helpText)
-     : helpText(helpText)
+    : editContent(new Edit::Text())
     {
+      editContent->SetText(helpText);
+      editContent->Disable();
+
       SetExitAction(GetClosedAction());
     }
 
@@ -40,7 +43,6 @@ namespace Lum {
     Lum::Object* Help::GetContent()
     {
       Edit::Edit  *edit;
-      Edit::Text  *editContent;
       Panel       *panel;
       View        *view;
 
@@ -48,10 +50,6 @@ namespace Lum {
       panel->SetMinHeight(Base::Size::workVRel,20);
       panel->SetHeight(Base::Size::workVRel,80);
 
-      editContent=new Edit::Text();
-      editContent->SetText(helpText);
-      editContent->Disable();
-
       view=new View();
       view->SetFlex(true,true);
 
