@@ -21,6 +21,7 @@
 
 #include <cassert>
 #include <cstdlib>
+#include <limits>
 
 #include <Lum/Base/Util.h>
 
@@ -90,6 +91,31 @@ namespace Lum {
       }
     }
 
+    void LineInfo::SetBackground(size_t x1, size_t x2,
+                                 const Style& normalStyle,
+                                 const Style& backgroundStyle)
+    {
+      size_t x;
+
+      Expand(x2);
+
+      /*
+        Find start position x1 in text.
+      */
+      x=x1-1; // Optimisation: x1 will be at x or later but never before
+      while (x<this->style.size() && pos[x]<x1) {
+        x++;
+      }
+
+      while (x<this->style.size() && pos[x]<=x2) {
+        if (this->style[x].bg==normalStyle.bg) {
+          this->style[x].bg=backgroundStyle.bg;
+        }
+
+        x++;
+      }
+    }
+
     void LineInfo::SetDecoration(size_t x1, unsigned long decoration)
     {
       size_t x;
@@ -109,8 +135,13 @@ namespace Lum {
 
     Config::Config(const std::wstring& name)
     : name(name),
-      createBackup(false),remTrailSpace(false),
-      showEOL(false),showTab(false),highlightTab(false),showSpace(false),
+      createBackup(false),
+      remTrailSpace(false),
+      showEOL(false),
+      showTab(false),
+      highlightTab(false),
+      showSpace(false),
+      showCurrentLine(false),
       syntax(syntaxRepository->Get(L"Standard"))
     {
       // no code
@@ -132,6 +163,7 @@ namespace Lum {
       showTab=config->showTab;
       highlightTab=config->highlightTab;
       showSpace=config->showSpace;
+      showCurrentLine=config->showCurrentLine;
       keys=config->keys;
       syntax=config->syntax;
     }
@@ -211,6 +243,16 @@ namespace Lum {
       showSpace=show;
     }
 
+    bool Config::ShowCurrentLine() const
+    {
+      return showCurrentLine;
+    }
+
+    void Config::SetShowCurrentLine(bool show)
+    {
+      showCurrentLine=show;
+    }
+
     void Config::SetSyntax(Syntax* syntax)
     {
       this->syntax=syntax;
@@ -294,8 +336,7 @@ namespace Lum {
     Edit::Edit()
     : config(NULL),
       hScrollOffset(3),
-      showCurrentLine(false),
-      horizOffset(0),
+      requiresKnob(true),
       font(OS::display->GetFont(OS::Display::fontTypeFixed)),
       drawBuffer(NULL),
       //font(OS::display->GetFont(OS::Display::fontTypeProportional)),
@@ -303,23 +344,21 @@ namespace Lum {
       actionFunctionExecuted(new Model::Action),
       actionResolveConfig(new Model::Action),
       colorSheme(NULL),
-      stableSelection(true),readWrite(true),
+      stableSelection(true),
+      readWrite(true),
       vSize(0),
       showBlockCursor(false)
     {
       SetBackground(OS::display->GetFill(OS::Display::listboxBackgroundFillIndex));
 
       SetClipFromParent(true);
-      //SetBackground(new OS::EmptyFill());
-
+      SetClipBackgroundToBorder(true);
+      
       SetCanFocus(true);
       RequestFocus();
       SetRequestsKeyboard(true);
 
       colorSheme=ColorSheme::GetDefaultColorSheme();
-
-      top=vAdjustment->GetTopModel();
-      first=hAdjustment->GetTopModel();
     }
 
     Edit::~Edit()
@@ -327,6 +366,24 @@ namespace Lum {
       delete drawBuffer;
     }
 
+    bool Edit::CanRefreshRedraw() const
+    {
+      if (refreshFull) {
+        return false;
+      }
+      
+      return true;
+    }
+
+    bool Edit::CanRefreshMove() const
+    {
+      if (refreshFull) {
+        return false;
+      }
+      
+      return true;
+    }
+
     bool Edit::CanExecuteAction(const Base::Datatype& type,
                                 Base::DataExchangeObject::ExchangeAction action) const
     {
@@ -420,6 +477,7 @@ namespace Lum {
         model->SetSyntax(config->GetSyntax());
       }
 
+      refreshFull=true;
       Redraw();
     }
 
@@ -448,6 +506,16 @@ namespace Lum {
       Scrollable::CalcSize();
     }
 
+    void Edit::SetRequiresKnobs(bool requiresKnob)
+    {
+      this->requiresKnob=requiresKnob;
+    }
+
+    bool Edit::RequiresKnobs() const
+    {
+      return requiresKnob;
+    }
+
     bool Edit::IsWriteable() const
     {
       return readWrite;
@@ -493,7 +561,7 @@ namespace Lum {
     */
     size_t Edit::GetLastVisiblePos() const
     {
-      return std::min(startLine->GetY()+vSize-1,model->GetLines());
+      return std::min(startLine->GetY()+height/font->height-1,model->GetLines());
     }
 
     /**
@@ -547,7 +615,7 @@ namespace Lum {
       GetLine(y,workCursor);
       GetLineInfo(workCursor,info);
 
-      xPos=-horizOffset;
+      xPos=-(hAdjustment->GetTop()-1);
       i=0;
       x=0;
       while (i<info.width.size() && xPos<cX) {
@@ -587,7 +655,7 @@ namespace Lum {
 
       i=0;
       pos=0;
-      cX=this->x-horizOffset;
+      cX=this->x-(hAdjustment->GetTop()-1);
       while (i<info.pos.size() && info.pos[i]<x) {
         pos=info.pos[i];
         cX+=info.width[i];
@@ -678,110 +746,30 @@ namespace Lum {
     /**
       Marks the given range of lines ([1..text.GetLines()]) as "needs to be refreshed".
     */
-    void Edit::SetDirtyRange(size_t y1, size_t y2)
+    void Edit::SetDirty(size_t line)
     {
-      if (y1>GetLastVisiblePos() || y2<GetFirstVisiblePos()) {
+      if (line>GetLastVisiblePos() || line<GetFirstVisiblePos()) {
         return;
       }
 
-      if (y1<GetFirstVisiblePos()) {
-        y1=GetFirstVisiblePos();
-      }
 
-      if (y2>GetLastVisiblePos()) {
-        y2=GetLastVisiblePos();
-      }
-
-      // Could use SetDirty and transformation of coords here...
-
-      for (size_t x=y1; x<=y2; x++) {
-        lines[x-GetFirstVisiblePos()].drawn=false;
-      }
-    }
-
-    void Edit::SetDirtyLine(size_t y)
-    {
-      SetDirtyRange(y,y);
-    }
-
-    /**
-      Mark everything as dirty.
-    */
-    void Edit::SetDirtyAll()
-    {
-      SetDirtyVisibleRange(1,vSize);
-    }
-
-
-    /**
-      Move the given range of lines (display relative) the given amount up
-      or down on the display.
-
-      NOTE
-      Only the lines array itself will be updated. All other elements must
-      be updated manually.
-    */
-    void Edit::MoveRegion(size_t start, size_t end, long count)
-    {
-      if (count==0) {
-        return;
-      }
-
-      std::vector<Line> linesBak;
-
-      if (moveCursor) {
-        SetDirtyLine(coy);
-        SetDirtyLine(cny);
-        moveCursor=false;
-      }
-
-      // Backup lines
-      linesBak.resize(lines.size());
-      for (size_t x=0; x<lines.size(); x++) {
-        linesBak[x]=lines[x];
-      }
-
-      if (count<0) {
-        for (size_t x=start-1; x<end; x++) {
-          linesBak[x].drawn=true;
-          linesBak[x+count]=lines[x];
-        }
-      }
-      else {
-        for (size_t x=end-1; x<start; x--) {
-          linesBak[x].drawn=true;
-          linesBak[x+count]=lines[x];
-        }
-      }
-
-      if (!moveRegion) {
-        moveRegion=true;
-        mrCount=count;
-        mrStart=start;
-        mrEnd=end;
-      }
-      else {
-        for (size_t x=start-1; x<end; x++) {
-          linesBak[x+count].drawn=false;
-        }
-      }
-
-      // Switch between backup and current lines
-      lines=linesBak;
+      lines[line-GetFirstVisiblePos()].drawn=false;
     }
 
     void Edit::MoveCursor(size_t ox, size_t oy, size_t nx, size_t ny)
     {
-      if (!moveCursor && !moveRegion) {
-        moveCursor=true;
-        cox=ox;
-        coy=oy;
-        cnx=nx;
-        cny=ny;
+      if (CanRefreshRedraw()) {       
+        if (oy==ny) {
+          refreshLines.push_back(oy);
+        }
+        else {
+          refreshLines.push_back(oy);
+          refreshLines.push_back(ny);
+        }
+        Redraw();
       }
       else {
-        SetDirtyLine(oy);
-        SetDirtyLine(ny);
+        refreshFull=true;
       }
     }
 
@@ -872,13 +860,8 @@ namespace Lum {
         info.pos.push_back(x+1);
       }
 
-      info.style.resize(info.text.size());
-      x=0;
-      while (x<info.style.size()) {
-        info.style[x]=colorSheme->styles[ColorSheme::normalStyle];
-
-        x++;
-      }
+      info.style.clear();
+      info.style.resize(info.text.size(),colorSheme->styles[ColorSheme::normalStyle]);
 
       info.token.clear();
       info.token.resize(info.pos.size(),NULL);
@@ -887,7 +870,7 @@ namespace Lum {
       info.width.clear();
       info.width.resize(info.pos.size());
       info.decoration.clear();
-      info.decoration.resize(info.pos.size());
+      info.decoration.resize(info.pos.size(),0);
     }
 
     void Edit::Colorize(size_t yPos,LineInfo& info)
@@ -900,8 +883,7 @@ namespace Lum {
         Syntax hilighting
       */
 
-      model->state.MoveToLine(yPos);
-      info.state=model->state.state;
+      info.state=model->state.MoveToLine(yPos);
       style=model->syntax->GetStyle(info.state);
 
       x=0;
@@ -969,14 +951,6 @@ namespace Lum {
         ++colorizer;
       }
 
-      /*
-        Coloring current line (optional)
-       */
-
-      if (showCurrentLine && cursor->GetY()==yPos) {
-        info.SetArea(1,info.orig.length()+1,colorSheme->styles[ColorSheme::selectionStyle]);
-      }
-
       /*
         Coloring of current selection block.
       */
@@ -1028,15 +1002,21 @@ namespace Lum {
         info.SetDecoration(cursor->GetX(),LineInfo::decoCursor);
       }
 
-      OS::FontExtent extent;
-      std::wstring   textPart(L" ");
+      /*
+        Coloring current line (optional)
+       */
+      if (config->ShowCurrentLine() && cursor->GetY()==yPos) {
+        info.SetBackground(1,info.pos[info.pos.size()-1],
+                           colorSheme->styles[ColorSheme::normalStyle],
+                           colorSheme->styles[ColorSheme::currentLineStyle]);
+      }
+
+      std::wstring textPart(L" ");
 
       for (size_t x=0; x<info.text.length(); x++) {
         textPart[0]=info.text[x];
 
-        font->StringExtent(textPart,extent,info.style[x].style);
-
-        info.width[x]=extent.width;
+        info.width[x]=font->StringWidth(textPart,info.style[x].style);
       }
     }
 
@@ -1078,14 +1058,14 @@ namespace Lum {
         textPart=info.text.substr(start,end-start+1);
 
         draw->PushForeground(info.style[start].bg);
-        draw->FillRectangle(this->x+xPos-horizOffset,
+        draw->FillRectangle(this->x+xPos-(hAdjustment->GetTop()-1),
                             this->y+(pos-1)*font->height,
                             width,font->height);
         draw->PopForeground();
 
         draw->PushForeground(info.style[start].fg);
         draw->PushFont(font,info.style[start].style);
-        draw->DrawString(this->x+xPos-horizOffset,
+        draw->DrawString(this->x+xPos-(hAdjustment->GetTop()-1),
                          this->y+(pos-1)*font->height+font->ascent,
                          textPart);
         draw->PopFont();
@@ -1093,15 +1073,19 @@ namespace Lum {
 
         if (info.decoration[start] & LineInfo::decoCursor) {
           draw->PushForeground(colorSheme->styles[ColorSheme::normalStyle].fg);
-          draw->DrawLine(this->x+xPos-horizOffset,this->y+(pos-1)*font->height,
-                         this->x+xPos-horizOffset,this->y+(pos-1)*font->height+font->height-1);
+          draw->DrawLine(this->x+xPos-(hAdjustment->GetTop()-1),
+                         this->y+(pos-1)*font->height,
+                         this->x+xPos-(hAdjustment->GetTop()-1),
+                         this->y+(pos-1)*font->height+font->height-1);
           draw->PopForeground();
         }
 
         if (info.decoration[start] & LineInfo::decoBlock) {
           draw->PushForeground(colorSheme->styles[ColorSheme::normalStyle].fg);
-          draw->DrawLine(this->x+xPos-horizOffset,this->y+(pos-1)*font->height+font->height-1,
-                         this->x+xPos-horizOffset+info.width[start]-1,this->y+(pos-1)*font->height+font->height-1);
+          draw->DrawLine(this->x+xPos-(hAdjustment->GetTop()-1),
+                         this->y+(pos-1)*font->height+font->height-1,
+                         this->x+xPos-(hAdjustment->GetTop()-1)+info.width[start]-1,
+                         this->y+(pos-1)*font->height+font->height-1);
           draw->PopForeground();
         }
       }
@@ -1109,7 +1093,7 @@ namespace Lum {
         xPos+=(x-info.pos[info.pos.size()-1]-1)*spaceWidth;
 
         draw->PushForeground(colorSheme->styles[ColorSheme::normalStyle].bg);
-        draw->FillRectangle(this->x+xPos-horizOffset,
+        draw->FillRectangle(this->x+xPos-(hAdjustment->GetTop()-1),
                             this->y+(pos-1)*font->height,
                             spaceWidth,font->height);
         draw->PopForeground();
@@ -1119,19 +1103,18 @@ namespace Lum {
     /**
       Print the given line at the given display position.
     */
-    void Edit::PrintLine(OS::DrawInfo* draw, LineInfo& info, size_t pos)
+    void Edit::PrintLine(OS::DrawInfo* draw, LineInfo& info, size_t pos, size_t line)
     {
-      size_t       a,p;
+      size_t       p;
       int          xPos;
       std::wstring textPart;
 
       // Draw areas of equal display parameter
       p=0;
-      xPos=-horizOffset;
+      xPos=-(hAdjustment->GetTop()-1);
       while (p<info.style.size()) {
         size_t width=0;
-
-        a=p;
+        size_t a=p;
 
         width=info.width[p];
 
@@ -1148,8 +1131,11 @@ namespace Lum {
         // TODO: Check if visible
 
         draw->PushForeground(info.style[p].bg);
-        draw->FillRectangle(this->x+xPos,this->y+(pos-1)*font->height,
-                            width,font->height);
+
+        draw->FillRectangle(this->x+xPos,
+                            this->y+(pos-1)*font->height,
+                            width,
+                            font->height);
         draw->PopForeground();
 
         draw->PushForeground(info.style[p].fg);
@@ -1165,7 +1151,7 @@ namespace Lum {
 
       // Draw potential decorations
 
-      xPos=-horizOffset;
+      xPos=-(hAdjustment->GetTop()-1);
       for (size_t x=0; x<info.text.length(); x++) {
         if (info.decoration[x] & LineInfo::decoCursor) {
           draw->PushForeground(colorSheme->styles[ColorSheme::normalStyle].fg);
@@ -1185,13 +1171,23 @@ namespace Lum {
       // Draw area behind last text
 
       if (xPos<=0) {
-        draw->PushForeground(colorSheme->styles[ColorSheme::normalStyle].bg);
+        if (config->ShowCurrentLine() && cursor->GetY()==line) {
+          draw->PushForeground(colorSheme->styles[ColorSheme::currentLineStyle].bg);
+        }
+        else {
+          draw->PushForeground(colorSheme->styles[ColorSheme::normalStyle].bg);
+        }
         draw->FillRectangle(this->x,this->y+(pos-1)*font->height,
                             width,font->height);
         draw->PopForeground();
       }
       else if (xPos<(int)width) {
-        draw->PushForeground(colorSheme->styles[ColorSheme::normalStyle].bg);
+        if (config->ShowCurrentLine() && cursor->GetY()==line) {
+          draw->PushForeground(colorSheme->styles[ColorSheme::currentLineStyle].bg);
+        }
+        else {
+          draw->PushForeground(colorSheme->styles[ColorSheme::normalStyle].bg);
+        }
         draw->FillRectangle(this->x+xPos,this->y+(pos-1)*font->height,
                             width-xPos,font->height);
         draw->PopForeground();
@@ -1200,12 +1196,13 @@ namespace Lum {
 
     void Edit::UpdateDimensions()
     {
-      if (model.Valid() && model->GetLines()>0) {
-        vAdjustment->Set(GetFirstVisiblePos(),
-                         GetLastVisiblePos()-GetFirstVisiblePos()+1,
-                         model->GetLines());
+      if (model.Valid() &&
+          model->GetLines()>0) {
+        vAdjustment->SetDimension(height/font->height,
+                                  model->GetLines());
 
-        hAdjustment->Set(horizOffset+1,width,32000);
+        hAdjustment->SetDimension(width,32000);
+        
       }
       else {
         hAdjustment->SetInvalid();
@@ -1213,42 +1210,16 @@ namespace Lum {
       }
     }
 
-    /**
-      Update the display, when the given lines are deleted.
-    */
-    void Edit::UpdateDelete(size_t from, size_t count)
-    {
-      if (IsVisible(from) || IsVisible(from+count)) {
-        SetDirtyAll(); // TODO!
-      }
-    }
-
-    /**
-      Update the display, when the given lines are inserted.
-    */
-    void Edit::UpdateInsert(size_t from, size_t /*count*/)
-    {
-      if (IsVisible(from)) {
-       SetDirtyRange(from,model->GetLines());
-      }
-      // TODO: we can check if we can move parts of the display
-    }
-
     /**
       Refresh each to be refreshed line.
     */
-    void Edit::Update()
+    void Edit::Update(OS::DrawInfo* draw)
     {
-      if (!visible) {
+      if (!visible || !model.Valid()) {
         return;
       }
 
-      OS::DrawInfo *draw=GetDrawInfo();
-      size_t       x,y;
-      bool         error;
-
-      x=0;
-      if (model.Valid()) {
+#if 0
         if (moveRegion) {
           draw->CopyArea(this->x,                                   /* sX     */
                          this->y+(mrStart-1)*font->height,          /* sY     */
@@ -1258,76 +1229,48 @@ namespace Lum {
                          this->y+(mrStart-1+mrCount)*font->height); /* dY     */
           moveRegion=false;
         }
+#endif
+      draw->PushClip(this->x,this->y,this->width,this->height);
+      draw->PushFont(font);
 
-        draw->PushClip(this->x,this->y,this->width,this->height);
+      size_t x,y;
+      bool   error;
 
-        if (moveCursor) {
-          LineInfo info;
+      x=0;
+      y=startLine->GetY();
+      GetLine(y,workCursor);
+      error=false;
+      while (x<lines.size() && !error) {
+        LineInfo info;
 
-          if (IsVisible(coy)) {
-            GetLine(coy,workCursor);
-            GetLineInfo(workCursor,info);
-            PrintChar(draw,info,coy-GetFirstVisiblePos()+1,cox);
-          }
-          if (IsVisible(cny)) {
-            if (coy!=cny) { // Calculate LineInfo only if old and new line differ
-              GetLine(cny,workCursor);
-              GetLineInfo(workCursor,info);
-            }
-            PrintChar(draw,info,cny-GetFirstVisiblePos()+1,cnx);
-          }
+        if (!lines[x].drawn) {
+          GetLineInfo(workCursor,info);
+          PrintLine(draw,info,x+1,y);
+          lines[x].drawn=true;
 
-          moveCursor=false;
-        }
-
-        y=startLine->GetY();
-        GetLine(y,workCursor);
-        error=false;
-        while (x<lines.size() && !error) {
-          LineInfo info;
-
-          if (!lines[x].drawn) {
-            GetLineInfo(workCursor,info);
-            PrintLine(draw,info,x+1);
-            lines[x].drawn=true;
-
-            // If the state of this line has changed, we mark the next line as
-            // "to be drawn", too.
-            if (lines[x].state!=info.state) {
-              lines[x].state=info.state;
-              if (x<lines.size()-1) {
-                lines[x+1].drawn=false;
-              }
+          // If the state of this line has changed, we mark the next line as
+          // "to be drawn", too.
+          if (lines[x].state!=info.state) {
+            lines[x].state=info.state;
+            if (x<lines.size()-1) {
+              lines[x+1].drawn=false;
             }
           }
-
-          x++;
-          y++;
-
-          error=!workCursor->Down();
         }
 
-        draw->PopClip();
+        x++;
+        y++;
+
+        error=!workCursor->Down();
       }
 
-      // This code block should move to Refresh as soon as we have a global refresh trigger flag
-      // for it.
-
-      if ((GetLastVisiblePos()-GetFirstVisiblePos()+1)*font->height<height) {
-        OS::DrawInfo *draw=GetDrawInfo();
-
-        draw->PushForeground(colorSheme->styles[ColorSheme::normalStyle].bg);
-        draw->FillRectangle(this->x,this->y+(GetLastVisiblePos()-GetFirstVisiblePos()+1)*font->height,
-                            width,height-(GetLastVisiblePos()-GetFirstVisiblePos()+1)*font->height);
-        draw->PopForeground();
-      }
-
-      UpdateDimensions();
+      draw->PopFont();
+      draw->PopClip();
     }
 
     bool Edit::CanScrollLeft() const
     {
-      return horizOffset>0;
+      return hAdjustment->GetTop()>1;
     }
 
     bool Edit::CanScrollRight() const
@@ -1339,40 +1282,11 @@ namespace Lum {
     {
       assert(lines!=0);
 
-      long count;
-
-      // Moving visible area
-      count=0;
-      GetLine(GetLastVisiblePos(),workCursor);
-      if (lines>0) {
-        // paging down
-        while (count<lines && workCursor->Down() && startLine->Down()) {
-          count++;
-        }
+      if (lines<0) {
+        vAdjustment->DecTop(std::abs(lines));
       }
-      else /*if (lines<0)*/ {
-        // paging up
-        while (count>lines && startLine->Up() && workCursor->Up()) {
-          count--;
-        }
-      }
-
-      if (count==0) {
-        return;
-      }
-
-      // Updating display. Count=0 will be droped silently
-
-      if ((size_t)std::abs(count)>=vSize) {
-        SetDirtyAll();
-      }
-      else if (count>0) {
-        MoveRegion(count+1,vSize,-count);
-        SetDirtyVisibleRange(vSize-count+1,count);
-      }
-      else /*if (count<0)*/ {
-        MoveRegion(1,vSize+count,-count);
-        SetDirtyVisibleRange(1,-count);
+      else if (lines>0) {
+        vAdjustment->IncTop(lines);
       }
     }
 
@@ -1380,12 +1294,7 @@ namespace Lum {
     {
       assert(pixel!=0);
 
-      if (pixel<0 && (size_t)-pixel>horizOffset) {
-        pixel=-horizOffset;
-      }
-
-      horizOffset+=pixel;
-      SetDirtyAll();
+      hAdjustment->IncTop(pixel);
     }
 
     /**
@@ -1393,92 +1302,31 @@ namespace Lum {
     */
     void Edit::MakeCursorVisible(Cursor* cursor)
     {
-      long     lines;
       LineInfo info;
 
       /*
         Adjust vertical
       */
 
-      // TODO: In fact we can display positions outside [FirstVisiblePos,LastVisisblePos]
-      // without complete redraw. The real intervall is something like:
-      // [FirstVisiblePos-vSize-1,LastVisisblePos+vSize-1]
-
-      if (cursor->GetY()<GetFirstVisiblePos()) {
+      if ((cursor->GetY()<GetFirstVisiblePos() &&
+           GetFirstVisiblePos()-cursor->GetY()>vSize) ||
+          (cursor->GetY()>GetLastVisiblePos() &&
+           cursor->GetY()-GetLastVisiblePos()>vSize)) {
+        /*
+          Movement over more than one page, we 
+          center the line on display
+         */
+        vAdjustment->CenterOn(cursor->GetY());
+      }     
+      else if (cursor->GetY()<GetFirstVisiblePos()) {
         /* Up */
-        lines=cursor->GetY()-GetFirstVisiblePos();
-
-        startLine->Goto(cursor);
-        startLine->GotoColumn(1);
+        vAdjustment->SetTop(cursor->GetY());
       }
       else if (cursor->GetY()>GetLastVisiblePos()) {
         /* Down */
-        lines=cursor->GetY()-GetLastVisiblePos();
-
-        startLine->Goto(cursor);
-        startLine->GotoColumn(1);
-
-        // We try to place the cursor at the buttom of the visible display
-        size_t h=1;
-        while (h<vSize && startLine->Up()) {
-          h++;
-        }
+        vAdjustment->SetTop(cursor->GetY()-vSize+1);
       }
-      else {
-        /* cursor already in visible area */
-        lines=0;
-      }
-
-      if (lines!=0) {
-        if ((size_t)std::abs(lines)<vSize) {
-          /*
-            We have to move less than the size of the display.
-            Thus we can copy parts of the display content
-            and only manually draw newly visible lines.
-          */
-          if (lines>0) {
-            MoveRegion(lines+1,vSize,-lines);
-            SetDirtyVisibleRange(vSize-lines+1,lines);
-          }
-          else /* (lines<0)*/ {
-            MoveRegion(1,vSize+lines,-lines);
-            SetDirtyVisibleRange(1,-lines);
-          }
-        }
-        else {
-          /* We must completly redraw the display */
-          workCursor->Goto(startLine);
-
-          /* We try to center the display around the given point */
-          if (lines>0) {
-            /* Down */
-            size_t h;
-
-            // Move workCursor to last visible line, to assure that
-            // we do not center beyond text end
-            h=1;
-            while (h<vSize && workCursor->Down()) {
-              h++;
-            }
-
-            h=vSize;
-            while (h>vSize/2 && workCursor->Down() && startLine->Down()) {
-              h--;
-            }
-          }
-          else {
-            /* Up */
-            size_t h=1;
-
-            while (h<vSize/2 && startLine->Up()) {
-              h++;
-            }
-          }
-
-          SetDirtyAll();
-        }
-      }
-
+      
       /*
         Adjust horizontal
       */
@@ -1486,7 +1334,7 @@ namespace Lum {
       GetLine(cursor->GetY(),workCursor);
       GetLineInfo(workCursor,info);
 
-      int xPos=-horizOffset;
+      int xPos=-(hAdjustment->GetTop()-1);
 
       size_t a=0;
       while (a<info.pos.size() && info.pos[a]<cursor->GetX()) {
@@ -1494,23 +1342,21 @@ namespace Lum {
         a++;
       }
 
-      // Snap back to horizOffset=0 if we can show the cursor
+      // Snap back to first==1 if we can show the cursor
       // without an horizontal offset.
-      if (xPos-hScrollOffset*spaceWidth>=0 && xPos+horizOffset+hScrollOffset*spaceWidth<=width) {
-        if (horizOffset>0) {
-          horizOffset=0;
-          SetDirtyAll();
+      if (xPos-hScrollOffset*spaceWidth>=0 &&
+          xPos+(hAdjustment->GetTop()-1)+hScrollOffset*spaceWidth<=width) {
+        if (hAdjustment->GetTop()>1) {
+          hAdjustment->SetTop(1);
         }
       }
       else if (xPos-(int)(hScrollOffset*spaceWidth)<0) {
         // Cursor is left of left edge
-        horizOffset=xPos+horizOffset-hScrollOffset*spaceWidth-width/2;
-        SetDirtyAll();
+        hAdjustment->SetTop(xPos+(hAdjustment->GetTop()-1)-hScrollOffset*spaceWidth-width/2+1);
       }
       else if (xPos+hScrollOffset*spaceWidth>width) {
         // Cursor is right of right edge
-        horizOffset=xPos+horizOffset+hScrollOffset*spaceWidth-width/2;
-        SetDirtyAll();
+        hAdjustment->SetTop(xPos+(hAdjustment->GetTop()-1)+hScrollOffset*spaceWidth-width/2+1);
       }
     }
 
@@ -1678,14 +1524,30 @@ namespace Lum {
     {
       if (!showBlockCursor) {
         showBlockCursor=true;
-        SetDirtyLine(blockCursor->GetY());
+        
+        if (CanRefreshRedraw()) {       
+          refreshLines.push_back(blockCursor->GetY());
+        }
+        else {
+          refreshFull=true;
+        }
+        
+        Redraw();
       }
     }
 
     void Edit::HideBlock()
     {
       if (showBlockCursor) {
-        SetDirtyLine(blockCursor->GetY());
+        if (CanRefreshRedraw()) {       
+          refreshLines.push_back(blockCursor->GetY());
+        }
+        else {
+          refreshFull=true;
+        }
+        
+        Redraw();
+        
         showBlockCursor=false;
       }
     }
@@ -1726,10 +1588,6 @@ namespace Lum {
         markB=this->model->GetCursor(L"markB");
         blockCursor=this->model->GetOrCreateCursor(L"BlockCursor",0/*{T.cursorHidden}*/);
 
-        moveRegion=false;
-        moveCursor=false;
-        horizOffset=0;
-
         TriggerResolveConfigAction();
         TriggerPropertyChangedAction();
       }
@@ -1739,46 +1597,34 @@ namespace Lum {
       return this->model.Valid();
     }
 
-    void Edit::HandleResize(size_t height)
+    void Edit::GetDimension(size_t& width, size_t& height)
     {
-      if (height!=vSize) {
-        vSize=height;
-        ResizeLines();
-
-        workCursor->Goto(startLine);
-        size_t x=1;
-        while (x<vSize && workCursor->Down()) {
-          x++;
-        }
-
-        // We want the last line of the text to be the last line in display, if possible
-        x=vSize-x;
-        while (x>0 && startLine->Up()) {
-          x--;
-        }
-
-        SetDirtyAll();
+      if (model.Valid() && model->GetLines()>0) {
+        width=32000;
+        height=font->height*model->GetLines();
+      }
+      else {
+        width=0;
+        height=0;
       }
     }
 
-    void Edit::SupplyDoubleBufferBitmap()
+    void Edit::Layout()
     {
-      if (drawBuffer==NULL || drawBuffer->GetWidth()<width || drawBuffer->GetHeight()<font->height) {
-        delete drawBuffer;
-        drawBuffer=OS::driver->CreateBitmap(width,font->height);
-      }
+      UpdateDimensions();
+
+      Scrollable::Layout();
     }
 
-    void Edit::Draw(int x, int y, size_t w, size_t h)
+    void Edit::Draw(OS::DrawInfo* draw,
+                    int x, int y, size_t w, size_t h)
     {
-      Scrollable::Draw(x,y,w,h);
+      Scrollable::Draw(draw,x,y,w,h);
 
       if (!OIntersect(x,y,w,h)) {
         return;
       }
 
-      OS::DrawInfo *draw=GetDrawInfo();
-
       /* --- */
 
       if (!Intersect(x,y,w,h)) {
@@ -1786,21 +1632,141 @@ namespace Lum {
       }
 
       if (model.Valid()) {
-        size_t start,end;
+        if (refreshLines.empty() && !refreshMove && !refreshFull) {
+            size_t start,end;
 
-        HandleResize(height/font->height);
+            start=GetFirstVisiblePos()+(y-this->y)/font->height;
+            end=start+(h-1)/font->height;
+            
+            for (size_t i=start; i<=end; i++) {
+              refreshLines.push_back(i);
+            }
+        }
 
-        start=(y-this->y)/font->height+1;
-        end=std::min(GetLastVisiblePos()-GetFirstVisiblePos()+1,
-                     start+h/font->height+1);
+        if (height/font->height!=vSize) {
+          vSize=height/font->height;
+          ResizeLines();
 
-        SetDirtyVisibleRange(start,end);
-        Update();
+          MakeCursorVisible(cursor);
+          refreshFull=true;
+        }
+
+        if (refreshMove) {
+          if (refreshLastTop==std::numeric_limits<size_t>::max()) {
+            // adjustment changed from invalid to valid
+            refreshFull=true;
+          }
+          else if (refreshLastTop>=vAdjustment->GetTop()+vSize ||
+                   vAdjustment->GetTop()>=refreshLastTop+vSize) {
+            // Move is bigger than screen height       
+            refreshFull=true;
+          }
+        }
+      
+        if (refreshFull) {
+          // Resync startLine with vAdjustment->GetTop()          
+          if (vAdjustment->GetTop()>GetFirstVisiblePos()) {
+            GetLine(GetLastVisiblePos(),workCursor);
+            while (vAdjustment->GetTop()>GetFirstVisiblePos() && workCursor->Down() && startLine->Down()) {
+            }
+          }
+          else if (vAdjustment->GetTop()<GetFirstVisiblePos()) {
+            GetLine(GetLastVisiblePos(),workCursor);
+            while (vAdjustment->GetTop()<GetFirstVisiblePos() && startLine->Up() && workCursor->Up()) {
+            }
+          }
+          
+          SetDirtyVisibleRange(1,lines.size());
+          Update(draw);
+
+          if ((GetLastVisiblePos()-GetFirstVisiblePos()+1)*font->height<height) {
+            draw->PushForeground(colorSheme->styles[ColorSheme::normalStyle].bg);
+            draw->FillRectangle(this->x,
+                                this->y+(GetLastVisiblePos()-GetFirstVisiblePos()+1)*font->height,
+                                width,
+                                height-(GetLastVisiblePos()-GetFirstVisiblePos()+1)*font->height);
+            draw->PopForeground();
+          }
+          
+          refreshFull=false;
+          refreshMove=false;
+          refreshLines.clear();
+        }
+        
+        if (refreshMove) {
+          // Resync startLine with vAdjustment->GetTop()          
+          if (vAdjustment->GetTop()>GetFirstVisiblePos()) {
+            GetLine(GetLastVisiblePos(),workCursor);
+            while (vAdjustment->GetTop()>GetFirstVisiblePos() && workCursor->Down() && startLine->Down()) {
+            }
+          }
+          else if (vAdjustment->GetTop()<GetFirstVisiblePos()) {
+            GetLine(GetLastVisiblePos(),workCursor);
+            while (vAdjustment->GetTop()<GetFirstVisiblePos() && startLine->Up() && workCursor->Up()) {
+            }
+          }
+          
+          if (vAdjustment->GetTop()>refreshLastTop) {
+            draw->CopyArea(this->x,                                   
+                           this->y+(vAdjustment->GetTop()-refreshLastTop)*font->height,
+                           width,
+                           (vSize-(vAdjustment->GetTop()-refreshLastTop))*font->height,
+                           this->x,
+                           this->y);
+            SetDirtyVisibleRange(vSize-(vAdjustment->GetTop()-refreshLastTop)+1,vAdjustment->GetTop()-refreshLastTop);
+          }
+          else {
+            draw->CopyArea(this->x,                                   
+                           this->y,
+                           width,
+                           (vSize-(refreshLastTop-vAdjustment->GetTop()))*font->height,
+                           this->x,
+                           this->y+(refreshLastTop-vAdjustment->GetTop())*font->height);
+            SetDirtyVisibleRange(1,refreshLastTop-vAdjustment->GetTop());
+          }
+          
+          Update(draw);
+          
+          refreshMove=false;
+        }  
+        
+        if (!refreshLines.empty()) {
+          for (std::list<size_t>::const_iterator line=refreshLines.begin();
+               line!=refreshLines.end();
+               ++line) {          
+            SetDirty(*line);
+          }
+          
+          Update(draw);
+
+          if ((GetLastVisiblePos()-GetFirstVisiblePos()+1)*font->height<height) {
+            draw->PushForeground(colorSheme->styles[ColorSheme::normalStyle].bg);
+            draw->FillRectangle(this->x,
+                                this->y+(GetLastVisiblePos()-GetFirstVisiblePos()+1)*font->height,
+                                width,
+                                height-(GetLastVisiblePos()-GetFirstVisiblePos()+1)*font->height);
+            draw->PopForeground();
+          }
+          
+          refreshLines.clear();
+        }
       }
       else {
         draw->PushForeground(colorSheme->styles[ColorSheme::normalStyle].bg);
         draw->FillRectangle(this->x,this->y,this->width,this->height);
         draw->PopForeground();
+        
+        refreshFull=false;
+        refreshMove=false;
+        refreshLines.clear();
+      }
+      
+      // Reinitialize refresh tracking
+      if (!vAdjustment->IsValid()) {
+        refreshLastTop=std::numeric_limits<size_t>::max();
+      }
+      else {
+        refreshLastTop=GetFirstVisiblePos();
       }
     }
 
@@ -1895,7 +1861,8 @@ namespace Lum {
       if (IsMarkSet() || (IsSelected() && markA->GetPos()==markB->GetPos())) {
         ClearSelection();
       }
-      Update();
+      Redraw();
+      //Update();
       return true;
     }
     else if (event.type==OS::MouseEvent::move && event.IsGrabed()) {
@@ -1974,18 +1941,31 @@ namespace Lum {
         if ((cursorMsg=dynamic_cast<const Text::CursorMsg*>(&msg))!=NULL) {
           if (cursorMsg->cursor==cursor) {
             MoveCursor(cursorMsg->oldPos.x,cursorMsg->oldPos.y,
-                        cursorMsg->cursor->GetX(),cursorMsg->cursor->GetY());
+                       cursorMsg->cursor->GetX(),cursorMsg->cursor->GetY());
             MakeCursorVisible(cursorMsg->cursor);
           }
         }
         else if ((redrawMsg=dynamic_cast<const Text::RedrawMsg*>(&msg))!=NULL) {
-          SetDirtyRange(redrawMsg->from,redrawMsg->to);
+          if (CanRefreshRedraw()) {
+            for (size_t i=redrawMsg->from; i<=redrawMsg->to; i++) {
+              refreshLines.push_back(i);
+            }
+          }
+          else {
+            refreshFull=true;
+          }
+          
+          Redraw();
         }
         else if ((deleteMsg=dynamic_cast<const Text::DeleteMsg*>(&msg))!=NULL) {
-          UpdateDelete(deleteMsg->from,deleteMsg->count);
+          refreshFull=true;
+          Redraw();
+          SetRelayout();
         }
         else if ((insertMsg=dynamic_cast<const Text::InsertMsg*>(&msg))!=NULL) {
-          UpdateInsert(insertMsg->from,insertMsg->count);
+          refreshFull=true;
+          Redraw();
+          SetRelayout();
         }
         else if ((propertyChangedMsg=dynamic_cast<const Text::PropertyChangedMsg*>(&msg))!=NULL) {
           TriggerPropertyChangedAction();
@@ -1994,26 +1974,34 @@ namespace Lum {
           TriggerResolveConfigAction();
         }
         else if ((textChangedMsg=dynamic_cast<const Text::TextChangedMsg*>(&msg))!=NULL) {
-          // Do nothing
+          // ignored
         }
-        else if (visible) {
-          MakeCursorVisible(cursor);
-          SetDirtyAll();
-          Update();
+        else {
+          refreshFull=true;
+          Redraw();
+          SetRelayout();
+          
+          if (visible) {
+            MakeCursorVisible(cursor);
+          }          
         }
       }
-      else if (model==top) {
-        if (!top->IsNull() && top->Get()!=GetFirstVisiblePos()) {
-          Page(top->Get()-GetFirstVisiblePos());
-          ExecutePost();
+      else if (model==vAdjustment->GetTopModel()) {
+        if (CanRefreshMove()) {
+          refreshMove=true;
+          SetRelayout();
+          Redraw();
+        }
+        else {
+          refreshFull=true;
+          SetRelayout();
+          Redraw();
         }
       }
-      else if (model==first) {
-        if (!first->IsNull() && first->Get()-1!=horizOffset) {
-          horizOffset=first->Get()-1;
-          SetDirtyAll();
-          ExecutePost();
-        }
+      else if (model==hAdjustment->GetTopModel()) {
+        refreshFull=true;
+        Redraw();
+        SetRelayout();
       }
     }
 
@@ -2041,7 +2029,6 @@ namespace Lum {
     void Edit::ExecutePost()
     {
       TriggerFunctionExecutedAction();
-      Update();
     }
   }
 }
