@@ -35,7 +35,7 @@ namespace Lum {
 
   bool FuelGauge::SetModel(Base::Model* model)
   {
-    this->model=dynamic_cast<Model::Double*>(model);
+    this->model=dynamic_cast<Model::Number*>(model);
 
     Control::SetModel(this->model);
 
@@ -85,7 +85,8 @@ namespace Lum {
       return;
     }
 
-    double percent=(model->Get()-model->GetMin())/(model->GetMax()-model->GetMin());
+    double percent=(model->GetDouble()-model->GetMinAsDouble())/
+                   (model->GetMaxAsDouble()-model->GetMinAsDouble());
 
     if (width>=height) {
       Lum::OS::FontExtent extent;
