@@ -377,6 +377,15 @@ namespace Lum {
     SetFlex(true,false);
   }
 
+  MenuStrip::~MenuStrip()
+  {
+    for (std::list<PullDownMenu*>::const_iterator object=list.begin();
+         object!=list.end();
+         ++object) {
+      delete *object;
+    }
+  }
+
   /**
     Finds the menu entry (@otype{PullDownMenu}) that opens the given (sub)menu).
   */
@@ -466,17 +475,14 @@ namespace Lum {
     Return the (PullDownMenu) object currently under the mouse or
     NULL, if there is no child object under the mouse.
   */
-  Object* MenuStrip::GetMouseSelected() const
+  Object* MenuStrip::GetMouseSelected(const OS::MouseEvent& event) const
   {
-    std::list<PullDownMenu*>::const_iterator object;
-
-    object=list.begin();
-    while (object!=list.end()) {
-      if ((*object)->MouseIsIn()) {
+    for (std::list<PullDownMenu*>::const_iterator object=list.begin();
+         object!=list.end();
+         ++object) {
+      if ((*object)->PointIsIn(event)) {
         return *object;
       }
-
-      ++object;
     }
 
     return NULL;
@@ -667,8 +673,10 @@ namespace Lum {
   /* ---------- PullDownMenu stuff ------------------*/
 
   PullDownMenu::PullDownMenu()
-  : label(NULL),sc('\0'),
+  : label(NULL),
+    sc('\0'),
     shortcutAction(new Model::Action),
+    scAssigned(false),
     subMenu(NULL)
   {
     SetBackground(OS::display->GetFill(OS::Display::menuPulldownBackgroundFillIndex));
@@ -709,16 +717,6 @@ namespace Lum {
     minWidth=width;
     minHeight=height;
 
-    if (sc!=L'\0' && dynamic_cast<Dialog*>(GetWindow()->GetMaster())!=NULL) {
-      dynamic_cast<Dialog*>(GetWindow()->GetMaster())->RegisterShortcut(this,
-                                                                        OS::qualifierAlt,std::wstring(1,sc),
-                                                                        shortcutAction);
-    }
-
-    if (subMenu!=NULL) {
-      subMenu->SetParent(GetWindow());
-    }
-
     Object::CalcSize();
   }
 
@@ -749,6 +747,20 @@ namespace Lum {
 
     /* ---- */
 
+    if (!scAssigned) {
+      if (sc!=L'\0' && dynamic_cast<Dialog*>(GetWindow()->GetMaster())!=NULL) {
+        dynamic_cast<Dialog*>(GetWindow()->GetMaster())->RegisterShortcut(this,
+                                                                          OS::qualifierAlt,std::wstring(1,sc),
+                                                                          shortcutAction);
+      }
+
+      if (subMenu!=NULL) {
+        subMenu->SetParent(GetWindow());
+      }
+
+      scAssigned=true;
+    }
+
     draw->activated=IsMouseActive();
     label->Draw(draw,x,y,w,h);
     draw->activated=false;
@@ -1256,7 +1268,7 @@ namespace Lum {
 
   bool RadioItem::SetModel(Base::Model* model)
   {
-    this->model=dynamic_cast<Model::SizeT*>(model);
+    this->model=dynamic_cast<Model::Number*>(model);
 
     Control::SetModel(this->model);
 
@@ -1298,7 +1310,7 @@ namespace Lum {
 
     /* Image */
 
-    draw->selected=(model.Valid() && model->Get()==value);
+    draw->selected=(model.Valid() && model->GetUnsignedLong()==value);
     DrawImage(draw);
     draw->selected=false;
 
@@ -1600,9 +1612,18 @@ namespace Lum {
   /* ---------- Menu stuff ------------------*/
 
   Menu::Menu()
-  : window(OS::driver->CreateWindow()),parent(NULL),
-    popup(true),registered(false),reference(NULL),manualPos(false),xPos(0),yPos(0),
-    parentMenu(NULL),child(NULL),selected(NULL),strip(NULL)
+  : window(OS::driver->CreateWindow()),
+    parent(NULL),
+    popup(true),
+    registered(false),
+    reference(NULL),
+    manualPos(false),
+    xPos(0),
+    yPos(0),
+    parentMenu(NULL),
+    child(NULL),
+    selected(NULL),
+    strip(NULL)
   {
     window->SetMaster(this);
 
@@ -2104,14 +2125,24 @@ namespace Lum {
             */
             menu->Select(menu->GetSelected());
           }
-          else if (this->strip!=NULL && this->strip->MouseIsIn()) {
+          else if (this->strip!=NULL) {
+            // Transform to screen coordinates
+            mouse->x+=window->GetX();
+            mouse->y+=window->GetY();
+
+            // Transform to strip window coordinates
+            mouse->x-=this->strip->GetWindow()->GetX();
+            mouse->y-=this->strip->GetWindow()->GetY();
+
             /*
               If the mouse is over the strip, check if a new pulldown
-              menu has been selected. If, close the old one.
+              menu has been selected. If this is the case, close the old one.
             */
-            object=this->strip->GetMouseSelected();
-            if (object!=NULL && reference!=object) {
-              CloseAll();
+            if (this->strip->PointIsIn(*mouse)) {
+              object=this->strip->GetMouseSelected(*mouse);
+              if (object!=NULL && reference!=object) {
+                CloseAll();
+              }
             }
           }
         }
@@ -2122,7 +2153,9 @@ namespace Lum {
           */
           menu=GetMenuWithSelection();
 
-          if (menu!=NULL && menu->selected!=NULL && menu->selected->GetSubMenu()==NULL) {
+          if (menu!=NULL &&
+              menu->selected!=NULL &&
+              menu->selected->GetSubMenu()==NULL) {
             /*
               Deselect the last selected entry, because it is not
               selected by the mouse anymore.
@@ -2262,7 +2295,7 @@ namespace Lum {
   }
 
   void Menu::AddRadioItem(const std::wstring& label,
-                         Model::SizeT* model,
+                         Model::Number* model,
                          size_t value,
                          unsigned long qualifier,
                          const std::wstring& key)
