@@ -25,9 +25,9 @@ namespace Lum {
   namespace Model {
 
     Adjustment::Adjustment()
-    : top(new SizeT(1)),
-      visible(new SizeT(0)),
-      total(new SizeT(0)),
+    : top(new ULong()),
+      visible(new ULong()),
+      total(new ULong()),
       stepSize(1)
     {
       // no code
@@ -38,7 +38,7 @@ namespace Lum {
       top->On(notify);
       visible->On(notify);
       total->On(notify);
-      
+
       Model::On(notify);
     }
 
@@ -47,7 +47,7 @@ namespace Lum {
       top->Off();
       visible->Off();
       total->Off();
-      
+
       Model::Off();
     }
 
@@ -60,37 +60,37 @@ namespace Lum {
     bool Adjustment::IsValid() const
     {
       return !(top->IsNull() || visible->IsNull() || total->IsNull()
-               || visible->Get()==0
-               || visible->Get()>total->Get()
-               || top->Get()<1);
+               || visible->GetUnsignedLong()==0
+               || visible->GetUnsignedLong()>total->GetUnsignedLong()
+               || top->GetUnsignedLong()<1);
     }
 
-    size_t Adjustment::GetTop() const
+    unsigned long Adjustment::GetTop() const
     {
       if (top->IsNull()) {
         return 1;
       }
       else {
-        return top->Get();
+        return top->GetUnsignedLong();
       }
     }
 
-    size_t Adjustment::GetBottom() const
+    unsigned long Adjustment::GetBottom() const
     {
-      return top->Get()+visible->Get()-1;
+      return top->GetUnsignedLong()+visible->GetUnsignedLong()-1;
     }
 
-    size_t Adjustment::GetVisible() const
+    unsigned long Adjustment::GetVisible() const
     {
-      return visible->Get();
+      return visible->GetUnsignedLong();
     }
 
-    size_t Adjustment::GetTotal() const
+    unsigned long Adjustment::GetTotal() const
     {
-      return total->Get();
+      return total->GetUnsignedLong();
     }
 
-    size_t Adjustment::IsCompletelyVisible() const
+    bool Adjustment::IsCompletelyVisible() const
     {
       return GetVisible()==GetTotal();
     }
@@ -100,11 +100,13 @@ namespace Lum {
       post: if visible>=1 then new value is accepted.
       post: top will be corrected to fit visible and total
      */
-    void Adjustment::Set(size_t top, size_t visible, size_t total)
+    void Adjustment::Set(unsigned long top, unsigned long visible, unsigned long total)
     {
       bool t,v,a;
 
       if (visible==0 || total==0) {
+        SetInvalid();
+
         return;
       }
 
@@ -124,9 +126,9 @@ namespace Lum {
       // triggers it change, we first switch off the changed models, set all values
       // and then switch all changed models back on.
 
-      a=this->total->IsNull() || this->total->Get()!=total;
-      v=this->visible->IsNull() ||this->visible->Get()!=visible;
-      t=this->top->IsNull() ||this->top->Get()!=top;
+      a=this->total->IsNull() || this->total->GetUnsignedLong()!=total;
+      v=this->visible->IsNull() ||this->visible->GetUnsignedLong()!=visible;
+      t=this->top->IsNull() ||this->top->GetUnsignedLong()!=top;
 
       if (a) {
         this->total->Off();
@@ -183,10 +185,10 @@ namespace Lum {
       Set a new value for top.
       post: if top>=1 & top<=total then new value is accepted.
      */
-    void Adjustment::SetTop(size_t top)
+    void Adjustment::SetTop(unsigned long top)
     {
       assert(top>=1);
-      assert(top<=total->Get()-visible->Get()+1);
+      assert(top<=total->GetUnsignedLong()-visible->GetUnsignedLong()+1);
 
       Set(top,GetVisible(),GetTotal());
     }
@@ -196,9 +198,9 @@ namespace Lum {
       post: if visible>=1 then new value is accepted.
       post: top will be corrected to fit visible and total
      */
-    void Adjustment::SetDimension(size_t visible, size_t total)
+    void Adjustment::SetDimension(unsigned long visible, unsigned long total)
     {
-      size_t top;
+      unsigned long top;
 
       if (visible==0 || total==0) {
         return;
@@ -226,17 +228,17 @@ namespace Lum {
       Set(top,visible,total);
     }
 
-    SizeT* Adjustment::GetTopModel() const
+    Number* Adjustment::GetTopModel() const
     {
       return top.Get();
     }
 
-    SizeT* Adjustment::GetVisibleModel() const
+    Number* Adjustment::GetVisibleModel() const
     {
       return visible.Get();
     }
 
-    SizeT* Adjustment::GetTotalModel() const
+    Number* Adjustment::GetTotalModel() const
     {
       return total.Get();
     }
@@ -246,17 +248,17 @@ namespace Lum {
       IncTop(stepSize);
     }
 
-    void Adjustment::IncTop(size_t stepSize)
+    void Adjustment::IncTop(unsigned long stepSize)
     {
-      if (IsValid()) {
-        if (top->Get()+stepSize-1<=total->Get()-visible->Get()) {
-          top->Add(stepSize);
-          Notify();
-        }
-        else if (top->Get()<=total->Get()-visible->Get()) {
-          top->Set(total->Get()-visible->Get()+1);
-          Notify();
-        }
+      assert(IsValid());
+
+      if (top->GetUnsignedLong()+stepSize-1<=total->GetUnsignedLong()-visible->GetUnsignedLong()) {
+        top->Add(stepSize);
+        Notify();
+      }
+      else if (top->GetUnsignedLong()<=total->GetUnsignedLong()-visible->GetUnsignedLong()) {
+        top->Set(total->GetUnsignedLong()-visible->GetUnsignedLong()+1);
+        Notify();
       }
     }
 
@@ -265,95 +267,103 @@ namespace Lum {
       DecTop(stepSize);
     }
 
-    void Adjustment::DecTop(size_t stepSize)
+    void Adjustment::DecTop(unsigned long stepSize)
     {
-      if (IsValid()) {
-        if (top->Get()>=1+stepSize) {
-          top->Sub(stepSize);
-          Notify();
-        }
-        else if (top->Get()>1) {
-          top->Set(1);
-          Notify();
-        }
+      assert(IsValid());
+
+      if (top->GetUnsignedLong()>=1+stepSize) {
+        top->Sub(stepSize);
+        Notify();
+      }
+      else if (top->GetUnsignedLong()>1) {
+        top->Set(1);
+        Notify();
       }
     }
 
     void Adjustment::PageBack()
     {
-      if (IsValid()) {
-        if (top->Get()>visible->Get()) {
-          top->Sub(visible->Get());
-          Notify();
-        }
-        else if (top->Get()!=1) {
-          top->Set(1);
-          Notify();
-        }
+      assert(IsValid());
+
+      if (top->GetUnsignedLong()>visible->GetUnsignedLong()) {
+        top->Sub(visible->GetUnsignedLong());
+        Notify();
+      }
+      else if (top->GetUnsignedLong()!=1) {
+        top->Set(1);
+        Notify();
       }
     }
 
     void Adjustment::PageForward()
     {
-      if (IsValid()) {
-        if (top->Get()+visible->Get()<total->Get()-visible->Get()+1) {
-          top->Add(visible->Get());
-          Notify();
-        }
-        else if (top->Get()!=total->Get()-visible->Get()+1) {
-          top->Set(total->Get()-visible->Get()+1);
-          Notify();
-        }
+      assert(IsValid());
+
+      if (top->GetUnsignedLong()+visible->GetUnsignedLong()<total->GetUnsignedLong()-visible->GetUnsignedLong()+1) {
+        top->Add(visible->GetUnsignedLong());
+        Notify();
+      }
+      else if (top->GetUnsignedLong()!=total->GetUnsignedLong()-visible->GetUnsignedLong()+1) {
+        top->Set(total->GetUnsignedLong()-visible->GetUnsignedLong()+1);
+        Notify();
       }
     }
 
-    void Adjustment::MakeVisible(size_t pos)
+    void Adjustment::MakeVisible(unsigned long pos)
     {
-      if (pos>=1 && pos<=total->Get()) {
-        if (pos<top->Get()) {
+      assert(IsValid());
+
+      if (pos>=1 && pos<=total->GetUnsignedLong()) {
+        if (pos<top->GetUnsignedLong()) {
           top->Set(pos);
           Notify();
         }
-        else if (pos>top->Get()+visible->Get()-1) {
-          if (pos<visible->Get()) {
+        else if (pos>top->GetUnsignedLong()+visible->GetUnsignedLong()-1) {
+          if (pos<visible->GetUnsignedLong()) {
             top->Set(1);
           }
           else {
-            top->Set(pos-visible->Get()+1);
+            top->Set(pos-visible->GetUnsignedLong()+1);
           }
           Notify();
         }
       }
     }
 
-    void Adjustment::CenterOn(size_t pos)
+    void Adjustment::CenterOn(unsigned long pos)
     {
-      if (pos<=visible->Get()/2) {
+      assert(IsValid());
+
+      if (pos<=visible->GetUnsignedLong()/2) {
         ShowFirstPage();
       }
-      else if (pos>=total->Get()-visible->Get()/2+1) {
+      else if (pos>=total->GetUnsignedLong()-visible->GetUnsignedLong()/2+1) {
         ShowLastPage();
       }
       else {
-        SetTop(pos-visible->Get()/2);
+        SetTop(pos-visible->GetUnsignedLong()/2);
       }
     }
 
     void Adjustment::ShowFirstPage()
     {
+      assert(IsValid());
+
       SetTop(1);
     }
     void Adjustment::ShowLastPage()
     {
-      SetTop(total->Get()-visible->Get()+1);
+      assert(IsValid());
+
+      SetTop(total->GetUnsignedLong()-visible->GetUnsignedLong()+1);
     }
 
-    void Adjustment::SetStepSize(size_t stepSize)
+    void Adjustment::SetStepSize(unsigned long stepSize)
     {
       this->stepSize=stepSize;
     }
 
-    size_t Adjustment::GetStepSize() const
+    unsigned long Adjustment::GetStepSize() const
     {
       return stepSize;
     }
