@@ -337,6 +337,74 @@ namespace Lum {
         draw->PopForeground();
       }
 
+     class EditFill : public Fill
+      {
+      private:
+        Color colorA;
+        Color colorB;
+
+      public:
+        EditFill(Color colorA, Color colorB);
+
+        void Draw(DrawInfo* draw,
+                  int xOff, int yOff, size_t width, size_t height,
+                  int x, int y, size_t w, size_t h);
+      };
+
+      EditFill::EditFill(Color colorA, Color colorB)
+      : colorA(colorA),
+        colorB(colorB)
+      {
+        topBorder=2;
+        bottomBorder=2;
+        leftBorder=2;
+        rightBorder=2;
+      }
+
+      void EditFill::Draw(OS::DrawInfo* draw,
+                          int xOff, int yOff, size_t width, size_t height,
+                          int x, int y, size_t w, size_t h)
+      {
+        Color shine,shadow, halfShadow, halfShine;
+      
+        shine=dynamic_cast<DefaultTheme*>(display->GetTheme())->shineColor;
+        shadow=dynamic_cast<DefaultTheme*>(display->GetTheme())->shadowColor;
+        halfShadow=dynamic_cast<DefaultTheme*>(display->GetTheme())->halfShadowColor;
+        halfShine=dynamic_cast<DefaultTheme*>(display->GetTheme())->halfShineColor;
+       
+        if (draw->selected) {
+          draw->PushForeground(colorB);
+        }
+        else {
+          draw->PushForeground(colorA);
+        }
+
+        draw->FillRectangle(x,y,w,h);
+        draw->PopForeground();
+
+        if (!draw->selected) {
+          draw->PushForeground(shadow);
+          draw->DrawLine(x,y+h-1,x,y+1);
+          draw->DrawLine(x,y,x+w-1,y);
+          draw->PopForeground();
+          
+          draw->PushForeground(halfShadow);
+          draw->DrawLine(x+1,y+h-2,x+1,y+2);
+          draw->DrawLine(x+1,y+1,x+w-2,y+1);
+          draw->PopForeground();
+
+          draw->PushForeground(halfShine);
+          draw->DrawLine(x+w-1,y+1,x+w-1,y+h-1);
+          draw->DrawLine(x+w-1,y+h-1,x,y+h-1);
+          draw->PopForeground();
+          
+          draw->PushForeground(shine);
+          draw->DrawLine(x+w-2,y+2,x+w-2,y+h-2);
+          draw->DrawLine(x+w-2,y+h-2,x+1,y+h-2);
+          draw->PopForeground();
+        }
+      }
+      
       Theme::Theme(OS::Display *display)
       : OS::Theme(display),
         display(display)
@@ -1773,7 +1841,7 @@ namespace Lum {
 
         fill[Display::tableBackgroundFillIndex]=new SelectFill(fillColors[Display::tableBackgroundFillIndex],color[Display::fillColor]);
         fill[Display::tableBackground2FillIndex]=new SelectFill(fillColors[Display::tableBackground2FillIndex],color[Display::fillColor]);
-        fill[Display::entryBackgroundFillIndex]=new SelectFill(fillColors[Display::entryBackgroundFillIndex],color[Display::fillColor]);
+        fill[Display::entryBackgroundFillIndex]=new EditFill(fillColors[Display::entryBackgroundFillIndex],color[Display::fillColor]);
 
         fill[Display::buttonBackgroundFillIndex]=new FrameFill(FrameFill::button,fillColors[Display::buttonBackgroundFillIndex]);
         fill[Display::positiveButtonBackgroundFillIndex]=new FrameFill(FrameFill::button,fillColors[Display::positiveButtonBackgroundFillIndex]);
