@@ -23,8 +23,6 @@
 #include <Lum/OS/Theme.h>
 #include <Lum/Manager/Display.h>
 
-#include <iostream>
-
 namespace Lum {
   namespace OS {
     namespace Base {
@@ -53,6 +51,9 @@ namespace Lum {
         inEventLoop(false),
         shown(false),
         focus(false),
+        activateOnOpen(false),
+        hasMenuHint(false),
+        screenOrientationHint(screenOrientationAgnostic),
         x(0),y(0),width(0),height(0),
         minWidth(0),minHeight(0),
         maxWidth(32000),maxHeight(32000),
@@ -204,6 +205,27 @@ namespace Lum {
         this->type=type;
       }
 
+      void Window::SetHasMenuHint(bool hasMenu)
+      {
+        hasMenuHint=hasMenu;
+
+      }
+
+      bool Window::GetHasMenuHint() const
+      {
+        return hasMenuHint;
+      }
+
+      void Window::SetScreenOrientationHint(ScreenOrientationHint hint)
+      {
+        screenOrientationHint=hint;
+      }
+
+      Window::ScreenOrientationHint Window::GetScreenOrientationHint() const
+      {
+        return screenOrientationHint;
+      }
+
       void Window::SetTitle(const std::wstring& name)
       {
         if (!name.empty()) {
@@ -222,32 +244,15 @@ namespace Lum {
         }
       }
 
-      void Window::SetMinWidth(size_t width)
+      void Window::SetMinMaxSize(size_t minWidth,
+                                 size_t minHeight,
+                                 size_t maxWidth,
+                                 size_t maxHeight)
       {
-        if (!IsOpen()) {
-          minWidth=width;
-        }
-      }
-
-      void Window::SetMinHeight(size_t height)
-      {
-        if (!IsOpen()) {
-          minHeight=height;
-        }
-      }
-
-      void Window::SetMaxWidth(size_t width)
-      {
-        if (!IsOpen()) {
-          maxWidth=width;
-        }
-      }
-
-      void Window::SetMaxHeight(size_t height)
-      {
-        if (!IsOpen()) {
-          maxHeight=height;
-        }
+        this->minWidth=minWidth;
+        this->minHeight=minHeight;
+        this->maxWidth=maxWidth;
+        this->maxHeight=maxHeight;
       }
 
       void Window::SetPos(int x, int y)
