@@ -125,6 +125,11 @@ namespace Lum {
         // TODO
       }
 
+      void Window::TriggerDraw(int x, int y, size_t width, size_t height)
+      {
+        // TODO
+      }
+
       bool Window::Open(bool activate)
       {
         bool xManPos;
@@ -199,7 +204,7 @@ namespace Lum {
         draw=driver->CreateDrawInfo(this);
 
         TriggerMaped();
-        TriggerRedraw(0,0,width,height);
+        //TriggerRefresh(0,0,width,height);
         OnFocusIn();
         TriggerMouseIn();
         display->AddWindow(this);
