@@ -25,10 +25,12 @@
 #include <Lum/BooleanButton.h>
 #include <Lum/Button.h>
 #include <Lum/Dialog.h>
+#include <Lum/Float.h>
 #include <Lum/Menu.h>
 #include <Lum/Popup.h>
 #include <Lum/Tab.h>
 #include <Lum/ValueButton.h>
+#include <Lum/View.h>
 
 #include <Lum/OS/X11/Display.h>
 #include <Lum/OS/X11/Window.h>
@@ -48,13 +50,15 @@ namespace Lum {
         class MenuPopup : public Popup
         {
         private:
-          std::set<Model::Action*>  actions;
-          std::set<Model::Boolean*> booleans;
+          Def::Menu                   *def;
+          std::set<Lum::Base::Model*> excludes;
+          std::set<Model::Action*>    actions;
+          std::set<Model::Boolean*>   booleans;
 
         private:
-          void AddToPopupArray(Array* array,
-                               Def::Menu* def,
-                               const std::set<Lum::Base::Model*>& excludes)
+          void AddToPopup(Float* floatMenu,
+                          Def::Menu* def,
+                          const std::set<Lum::Base::Model*>& excludes)
           {
             for (std::list<Def::MenuItem*>::const_iterator iter=def->GetItems().begin();
                  iter!=def->GetItems().end();
@@ -64,12 +68,13 @@ namespace Lum {
               if (dynamic_cast<Def::Menu*>(*iter)!=NULL) {
                 Def::Menu *sub=dynamic_cast<Def::Menu*>(*iter);
 
-                AddToPopupArray(array,sub,excludes);
+                AddToPopup(floatMenu,sub,excludes);
               }
               else if (dynamic_cast<Def::MenuAction*>(*iter)!=NULL) {
                 Def::MenuAction *action=dynamic_cast<Def::MenuAction*>(*iter);
 
-                if (excludes.find(action->GetAction().GetAction())==excludes.end()) {
+                if (excludes.find(action->GetAction().GetAction())==excludes.end() &&
+                  action->GetAction().GetAction()->IsEnabled()) {
                   actions.insert(action->GetAction().GetAction());
                   ObservePrior(action->GetAction().GetAction());
 
@@ -77,13 +82,14 @@ namespace Lum {
                                                 action->GetAction().GetAction(),
                                                 true,false);
 
-                  array->Add(button);
+                  floatMenu->Add(button);
                 }
               }
               else if (dynamic_cast<Def::MenuBoolean*>(*iter)!=NULL) {
                 Def::MenuBoolean *boolean=dynamic_cast<Def::MenuBoolean*>(*iter);
 
-                if (excludes.find(boolean->GetBoolean().GetBoolean())==excludes.end()) {
+                if (excludes.find(boolean->GetBoolean().GetBoolean())==excludes.end() &&
+                    boolean->GetBoolean().GetBoolean()->IsEnabled()) {
                   Def::Boolean b=boolean->GetBoolean();
 
                   booleans.insert(b.GetBoolean());
@@ -91,7 +97,7 @@ namespace Lum {
 
                   Lum::Object *object=Manager::Behaviour::Instance()->GetBooleanControl(b,true,false);
 
-                  array->Add(object);
+                  floatMenu->Add(object);
                 }
               }
               else if (dynamic_cast<Def::MenuOneOfN*>(*iter)!=NULL) {
@@ -105,21 +111,14 @@ namespace Lum {
           }
 
         public:
-          MenuPopup(Def::Menu* def,
-                    const std::set<Lum::Base::Model*>& excludes)
+          MenuPopup()
+          : def(NULL)
           {
-            Array        *array = new Array();
             X11::Display *display=dynamic_cast<X11::Display*>(OS::display);
 
-            array->SetFlex(true,true);
-            array->SetSpace(true,true);
-            array->SetHorizontalCount(2);
-
-            AddToPopupArray(array,def,excludes);
-
-            SetMain(array,false,true);
             SetBackground(dynamic_cast<X11::GtkTheme*>(OS::display->GetTheme())->appMenuBackground);
             SetType(OS::Window::typeCustom);
+            GetWindow()->SetScreenOrientationHint(OS::Window::screenOrientationBothSupported);
             dynamic_cast<X11::Window*>(GetWindow())->SetCustomWindowType(XInternAtom(display->display,
                                                                          "_HILDON_WM_WINDOW_TYPE_APP_MENU",
                                                                          false));
@@ -127,10 +126,48 @@ namespace Lum {
             Observe(GetClosedAction());
           }
 
+          ~MenuPopup()
+          {
+            // no code
+          }
+
+          void SetMenu(Def::Menu* def,
+                       const std::set<Lum::Base::Model*>& excludes)
+          {
+            this->def=def;
+
+            this->excludes=excludes;
+          }
+
+          void PreInit()
+          {
+            Float *floatMenu=new Float();
+            View  *view=new View();
+
+            floatMenu->SetFlex(true,true);
+
+            if (display->IsLandscape()) {
+              floatMenu->SetPreferredColumns(2);
+            }
+            else {
+              floatMenu->SetPreferredColumns(1);
+            }
+
+            AddToPopup(floatMenu,def,excludes);
+
+            view->SetFlex(true,true);
+            view->SetObject(floatMenu);
+
+            SetMain(view,false,true);
+
+            Popup::PreInit();
+          }
+
           void Resync(Lum::Base::Model* model, const Lum::Base::ResyncMsg& msg)
           {
-            if (model==GetClosedAction() && GetClosedAction()->IsFinished()) {
-              Hide();
+            if (model==GetClosedAction() &&
+                GetClosedAction()->IsFinished()) {
+              Close();
               return;
             }
             else if (dynamic_cast<Model::Action*>(model)!=NULL) {
@@ -141,9 +178,9 @@ namespace Lum {
                     action->IsFinished() &&
                     IsOpen() &&
                     IsShown()) {
-                  Hide();
-                  return;
+                  Close();
                 }
+                return;
               }
             }
             else if (dynamic_cast<Model::Boolean*>(model)!=NULL) {
@@ -153,9 +190,9 @@ namespace Lum {
                 if (boolean->IsEnabled() &&
                     IsOpen() &&
                     IsShown()) {
-                  Hide();
-                  return;
+                  Close();
                 }
+                return;
               }
             }
 
@@ -164,7 +201,6 @@ namespace Lum {
         };
 
       private:
-        Menu      *menu;
         MenuPopup *popup;
 
       private:
@@ -208,81 +244,35 @@ namespace Lum {
           }
         }
 
-        bool CanBeDisplayedAsButtonMenuPopup(Def::Menu* def,
-                                             const std::set<Lum::Base::Model*>& excludes)
-        {
-          size_t actions=0;
-          size_t bools=0;
-          size_t oneOfNs=0;
-
-          size_t buttonrows=0;
-
-          CountModels(def,actions,bools,oneOfNs,excludes);
-
-          if ((actions+bools) % columns!=0) {
-            buttonrows=(actions+bools) / rows + 1;
-          }
-          else {
-            buttonrows=(actions+bools) / columns;
-          }
-
-          buttonrows+=oneOfNs;
-
-          return buttonrows<rows;
-        }
-
       public:
         CustomMenuHandlerPopup(OS::Window* window)
         : Manager::Behaviour::CustomMenuHandler(window),
-          menu(NULL),
           popup(NULL)
         {
         }
 
         ~CustomMenuHandlerPopup()
         {
-          delete menu;
           delete popup;
         }
 
         bool SetMenu(Def::Menu* def,
                 const std::set<Lum::Base::Model*>& excludes)
         {
-          delete menu;
-          delete popup;
-
-          menu=NULL;
-          popup=NULL;
-
-          if (CanBeDisplayedAsButtonMenuPopup(def,excludes)) {
-            popup=new MenuPopup(def,excludes);
+          if (popup==NULL) {
+            popup=new MenuPopup();
             popup->SetParent(window);
+          }
 
-          }
-          else {
-            menu=ConvertToMenu(def,excludes);
-            menu->SetParent(window);
-          }
+          popup->SetMenu(def,excludes);
 
           return true;
         }
 
         bool Open()
         {
-          if (menu!=NULL) {
-            menu->SetPos(window->GetX()+50,window->GetY()+50);
-            return menu->OpenInStickyMode();
-          }
-          else if (popup!=NULL) {
-            if (popup->IsOpen()) {
-              return popup->Show();
-            }
-            else if (popup->Open()) {
-              return true;
-            }
-            else {
-              return false;
-            }
+          if (popup!=NULL) {
+            return popup->Open();
           }
           else {
             return false;
@@ -310,6 +300,11 @@ namespace Lum {
         bool SetMenu(Def::Menu* def,
                 const std::set<Lum::Base::Model*>& excludes)
         {
+          if (menu!=NULL) {
+            if (menu->IsOpen()) {
+              menu->Close();
+            }
+          }
           delete menu;
 
           menu=ConvertToMenu(def,excludes);
@@ -359,22 +354,22 @@ namespace Lum {
         }
       };
 
-      class MultiViewArray : public Array
+      class MultiViewFloat : public Float
       {
       private:
         Def::MultiView                    multiView;
-        Model::SizeTRef                   value;
+        Model::NumberRef                  value;
         std::vector<Model::ActionRef>     actions;
         std::vector<Def::MultiView::View> views;
         std::vector<Dialog*>              dialogs;
 
       public:
-        MultiViewArray(const Def::MultiView& multiView)
+        MultiViewFloat(const Def::MultiView& multiView)
          : multiView(multiView),
            value(multiView.GetValue())
         {
           if (!value.Valid()) {
-            value=new Model::SizeT();
+            value=new Model::ULong();
           }
 
           Observe(value);
@@ -390,14 +385,14 @@ namespace Lum {
             views.push_back(*view);
             Observe(action);
 
-            button=Button::Create(view->GetDesc().GetLabel(),action,true,true);
-            button->SetMaxHeight(Lum::Base::Size::pixel,105); //Shouldn't be hardcoded
+            button=Button::Create(view->GetDesc().GetLabel(),action,true,false);
+            button->SetHeight(Lum::Base::Size::pixel,105); //Shouldn't be hardcoded
 
             Add(button);
           }
         }
 
-        ~MultiViewArray()
+        ~MultiViewFloat()
         {
           for (std::vector<Dialog*>::iterator dialog=dialogs.begin();
                dialog!=dialogs.end();
@@ -406,25 +401,25 @@ namespace Lum {
           }
         }
 
-        void CalcSize()
+        void Layout()
         {
           assert(GetWindow()!=NULL);
           Observe(GetWindow()->GetOpenedAction());
 
-          Array::CalcSize();
+          Float::Layout();
         }
 
         void ShowDialog()
         {
-          if (dialogs[value->Get()]==NULL) {
+          if (dialogs[value->GetUnsignedLong()]==NULL) {
             Def::Menu *menu=NULL;
 
             if (multiView.GetPreMenu()!=NULL) {
               menu=new Def::Menu(*multiView.GetPreMenu());
 
-              if (views[value->Get()].GetMenu()!=NULL) {
-                menu->Group(views[value->Get()].GetDesc())
-                ->Append(views[value->Get()].GetMenu())
+              if (views[value->GetUnsignedLong()].GetMenu()!=NULL) {
+                menu->Group(views[value->GetUnsignedLong()].GetDesc())
+                ->Append(views[value->GetUnsignedLong()].GetMenu())
                 ->End();
               }
 
@@ -433,12 +428,12 @@ namespace Lum {
               }
             }
 
-            dialogs[value->Get()]=new ViewDialog(views[value->Get()].GetObject());
-            dialogs[value->Get()]->SetParent(GetWindow());
-            dialogs[value->Get()]->SetMenu(menu);
+            dialogs[value->GetUnsignedLong()]=new ViewDialog(views[value->GetUnsignedLong()].GetObject());
+            dialogs[value->GetUnsignedLong()]->SetParent(GetWindow());
+            dialogs[value->GetUnsignedLong()]->SetMenu(menu);
           }
 
-          Dialog* dialog=dialogs[value->Get()];
+          Dialog* dialog=dialogs[value->GetUnsignedLong()];
 
           if (dialog->Open()) {
             dialog->EventLoop();
@@ -454,7 +449,7 @@ namespace Lum {
               model==GetWindow()->GetOpenedAction() &&
               GetWindow()->GetOpenedAction()->IsFinished() && !value->IsNull()) {
 
-            Array::Resync(model,msg);
+            Float::Resync(model,msg);
 
             ShowDialog();
 
@@ -473,7 +468,7 @@ namespace Lum {
             ShowDialog();
           }
 
-          Array::Resync(model,msg);
+          Float::Resync(model,msg);
         }
       };
 #endif
@@ -505,9 +500,22 @@ namespace Lum {
                                         const Def::MultiView& multiView) const
       {
 #if HILDON_CHECK_VERSION(2,1,0)
-        Array     *array=new MultiViewArray(multiView);
+        Float     *floatMenu=new MultiViewFloat(multiView);
+        View      *view=new View();
         Def::Menu *menu=NULL;
 
+        //dialog->GetWindow()->SetScreenOrientationHint(Window::screenOrientationBothSupported);
+
+        if (display->IsLandscape()) {
+          floatMenu->SetPreferredColumns(3);
+        }
+        else {
+          floatMenu->SetPreferredColumns(2);
+        }
+
+        view->SetFlex(true,true);
+        view->SetObject(floatMenu);
+
         if (multiView.GetPreMenu()!=NULL) {
           menu=new Def::Menu(*multiView.GetPreMenu());
 
@@ -518,11 +526,7 @@ namespace Lum {
           }
         }
 
-        array->SetFlex(true,true);
-        array->SetSpace(true,true);
-        array->SetHorizontalCount(std::min(multiView.GetViews().size(),(size_t)3));
-
-        dialog->SetMain(array);
+        dialog->SetMain(view);
         dialog->SetMenu(menu);
 #else
         Tab                  *tab=Tab::Create(true,true);
