@@ -47,11 +47,24 @@ namespace Lum {
       return drawsFocused;
     }
 
-    void EmptyImage::Draw(OS::DrawInfo *draw, int x, int y, size_t w, size_t h)
+    void EmptyImage::Draw(OS::DrawInfo *draw, int x, int y)
     {
       // no code
     }
 
+    void EmptyImage::DrawScaled(OS::DrawInfo *draw,
+                                int x, int y,
+                                size_t w, size_t h)
+    {
+      // no code
+    }
+
+    void EmptyImage::DrawStretched(OS::DrawInfo *draw,
+                                   int x, int y,
+                                   size_t w, size_t h)
+    {
+      // no code
+    }
   }
 }
 
