@@ -24,10 +24,17 @@
 
 #include <uxtheme.h>
 #include <tmschema.h>
+
+#include <Lum/Private/Config.h>
+
 #include <Lum/OS/Win32/OSAPI.h>
 
 #include <Lum/Base/Size.h>
 
+#if defined(LUM_HAVE_LIB_CAIRO)
+  #include <Lum/OS/Cairo/DrawInfo.h>
+#endif
+
 #include <Lum/OS/Win32/Display.h>
 #include <Lum/OS/Win32/DrawInfo.h>
 
@@ -414,10 +421,23 @@ namespace Lum {
 
       void ThemeImage::DrawStretched(Lum::OS::DrawInfo *draw, int x, int y, size_t w, size_t h)
       {
-        HDC  dc=dynamic_cast<DrawInfo*>(draw)->dc;
+        HDC  dc;
         RECT rect;
         int  style;
 
+        if (dynamic_cast<DrawInfo*>(draw)!=NULL) {
+          dc=dynamic_cast<DrawInfo*>(draw)->dc;
+        }
+#if defined(LUM_HAVE_LIB_CAIRO)
+        else if (dynamic_cast<Cairo::DrawInfo*>(draw)!=NULL) {
+          cairo_surface_flush(dynamic_cast<Cairo::DrawInfo*>(draw)->surface);
+          dc=cairo_win32_surface_get_dc(dynamic_cast<Cairo::DrawInfo*>(draw)->surface);
+        }
+#endif        
+        else {
+          assert(false);
+        }
+        
         rect.left=x;
         rect.top=y;
         rect.right=x+w;
@@ -958,6 +978,12 @@ namespace Lum {
         }
 
         SetTextAlign(dc,TA_BASELINE+TA_LEFT);
+        
+#if defined(LUM_HAVE_LIB_CAIRO)
+        if (dynamic_cast<Cairo::DrawInfo*>(draw)!=NULL) {
+          cairo_surface_mark_dirty(dynamic_cast<Cairo::DrawInfo*>(draw)->surface);
+        }
+#endif        
       }
 
 
@@ -989,6 +1015,7 @@ namespace Lum {
             topBorder=GetSystemMetrics(SM_CYEDGE);
             bottomBorder=topBorder;
           }
+          
           // 1 pixel for the focus frame and two pixel for the space between frame and content
           leftBorder+=1+2;
           rightBorder+=1+2;
@@ -1284,40 +1311,61 @@ namespace Lum {
                       int xOff, int yOff, size_t width, size_t height,
                       int x, int y, size_t w, size_t h)
       {
+        Bitmap *bitmap;
         HBRUSH brush;
-        HDC    dc=dynamic_cast<DrawInfo*>(draw)->dc;
-        RECT   rect;
+        HDC    dc=0;
+        RECT   content,clip;
         int    style;
+        bool   fillRect=true;
 
-        rect.left=x;
-        rect.top=y;
-        rect.right=x+w;
-        rect.bottom=y+h;
+        bitmap=driver->CreateBitmap(width,height);
+        if (bitmap==NULL) {
+          std::cerr << "Cannot create bitmap" << std::endl;
+          return;
+        }
+
+        if (transparentBorder) {
+          draw->CopyToBitmap(xOff,yOff,width,height,0,0,bitmap);
+        }
+        
+#if defined(LUM_HAVE_LIB_CAIRO)
+        if (dynamic_cast<Cairo::Win32DrawInfo*>(draw)!=NULL) {
+          cairo_surface_flush(dynamic_cast<Cairo::DrawInfo*>(bitmap->GetDrawInfo())->surface);
+          dc=cairo_win32_surface_get_dc(dynamic_cast<Cairo::DrawInfo*>(bitmap->GetDrawInfo())->surface);
+        }
+#endif
+        if (dynamic_cast<DrawInfo*>(draw)!=NULL) {
+          dc=dynamic_cast<DrawInfo*>(bitmap->GetDrawInfo())->dc;
+        }
+        else {
+          assert(true);
+        }
+
+        // Original background area
+        clip.left=x-xOff;
+        clip.top=y-yOff;
+        clip.right=content.left+w;
+        clip.bottom=content.top+h;
+        
+        // Area the background area should be cliped to
+        content.left=0;
+        content.right=width;
+        content.top=0;
+        content.bottom=height;
 
         switch (fill) {
         case Display::backgroundFillIndex:
-          /*if (HasTheming()) {
-            RECT content,extent,clip;
-
-            content.left=xOff;
-            content.right=xOff+width;
-            content.top=yOff;
-            content.bottom=yOff+height;
-
+          if (HasTheming()) {
+            RECT extent;
+          
             MyGetThemeBackgroundExtent(windowTheme,dc,WP_DIALOG,1,&content,&extent);
-
-            clip.left=x;
-            clip.right=x+w;
-            clip.top=y;
-            clip.bottom=y+h;
-
             MyDrawThemeBackground(windowTheme,dc,WP_DIALOG,1,&extent,&clip);
 
             return;
           }
-          else {*/
+          else {
             brush=::GetSysColorBrush(COLOR_3DFACE);
-          //}
+          }
           break;
         case Display::tableBackgroundFillIndex:
         case Display::tableBackground2FillIndex:
@@ -1353,8 +1401,6 @@ namespace Lum {
         case Display::negativeButtonBackgroundFillIndex:
         case Display::defaultButtonBackgroundFillIndex:
           if (HasTheming()) {
-            RECT content,clip;
-
             if (draw->disabled) {
               style=PBS_DISABLED;
             }
@@ -1371,52 +1417,32 @@ namespace Lum {
               style=PBS_NORMAL;
             }
 
-            content.left=xOff;
-            content.right=xOff+width;
-            content.top=yOff;
-            content.bottom=yOff+height;
-
-            clip.left=x;
-            clip.right=x+w;
-            clip.top=y;
-            clip.bottom=y+h;
-
             MyDrawThemeBackground(buttonTheme,dc,BP_PUSHBUTTON,style,&content,&clip);
 
-            if (draw->focused) {
-              xOff+=leftBorder-3;
-              yOff+=topBorder-3;
-              width-=leftBorder+rightBorder-6;
-              height-=topBorder+bottomBorder-6;
-
-              draw->PushForeground(GetSysColor(COLOR_BTNTEXT));
-              draw->PushDash("\001\001",2,::Lum::OS::DrawInfo::fDash);
-              draw->DrawLine(xOff        ,yOff,         xOff+width-1,yOff);
-              draw->DrawLine(xOff+width-1,yOff,         xOff+width-1,yOff+height-1);
-              draw->DrawLine(xOff+width-1,yOff+height-1,xOff,        yOff+height-1);
-              draw->DrawLine(xOff        ,yOff+height-1,xOff,        yOff);
-              draw->PopDash();
-              draw->PopForeground();
+            if (draw->focused && !draw->disabled) {
+              RECT rect2;
+              
+              rect2.left=content.left+leftBorder-3;
+              rect2.right=content.right-rightBorder+3;
+              rect2.top=content.top+topBorder-3;
+              rect2.bottom=content.bottom-bottomBorder+3;
+              
+              /* ignore */ DrawFocusRect(dc,&rect2);
             }
 
-            return;
+            fillRect=false;
           }
           else {
-            rect.left=xOff;
-            rect.top=yOff;
-            rect.right=xOff+width;
-            rect.bottom=yOff+height;
-
             /* ignore */ DrawFrameControl(dc,
-                                          &rect,
+                                          &content,
                                           DFC_BUTTON,
                                           DFCS_BUTTONPUSH | (draw->selected ? DFCS_PUSHED : 0));
-            return;
+            fillRect=false;
           }
           break;
         case Display::scrollButtonBackgroundFillIndex:
           if (HasTheming()) {
-            RECT content,extent,clip;
+            RECT extent;
 
             if (draw->disabled) {
               style=PBS_DISABLED;
@@ -1434,21 +1460,10 @@ namespace Lum {
               style=PBS_NORMAL;
             }
 
-            content.left=xOff;
-            content.right=xOff+width;
-            content.top=yOff;
-            content.bottom=yOff+height;
-
             MyGetThemeBackgroundExtent(buttonTheme,dc,BP_PUSHBUTTON,style,&content,&extent);
-
-            clip.left=x;
-            clip.right=x+w;
-            clip.top=y;
-            clip.bottom=y+h;
-
             MyDrawThemeBackground(buttonTheme,dc,BP_PUSHBUTTON,style,&extent,&clip);
 
-            return;
+            fillRect=false;
           }
           else {
             brush=::GetSysColorBrush(COLOR_BTNFACE);
@@ -1456,8 +1471,6 @@ namespace Lum {
           break;
         case Display::comboBackgroundFillIndex:
           if (HasTheming()) {
-            RECT content,clip;
-
             if (draw->disabled) {
               style=PBS_DISABLED;
             }
@@ -1474,67 +1487,40 @@ namespace Lum {
               style=PBS_NORMAL;
             }
 
-            content.left=xOff;
-            content.right=xOff+width;
-            content.top=yOff;
-            content.bottom=yOff+height;
-
-            clip.left=x;
-            clip.right=x+w;
-            clip.top=y;
-            clip.bottom=y+h;
-
             MyDrawThemeBackground(buttonTheme,dc,BP_PUSHBUTTON,style,&content,&clip);
 
-            if (draw->focused) {
-              xOff+=leftBorder-2;
-              yOff+=topBorder-2;
-              width-=leftBorder+rightBorder-4;
-              height-=topBorder+bottomBorder-4;
-
-              draw->PushForeground(GetSysColor(COLOR_BTNTEXT));
-              draw->PushDash("\001\001",2,::Lum::OS::DrawInfo::fDash);
-              draw->DrawLine(xOff        ,yOff,         xOff+width-1,yOff);
-              draw->DrawLine(xOff+width-1,yOff,         xOff+width-1,yOff+height-1);
-              draw->DrawLine(xOff+width-1,yOff+height-1,xOff,        yOff+height-1);
-              draw->DrawLine(xOff        ,yOff+height-1,xOff,        yOff);
-              draw->PopDash();
-              draw->PopForeground();
+            if (draw->focused && !draw->disabled) {
+              RECT rect2;
+              
+              rect2.left=content.left+leftBorder-2;
+              rect2.right=content.right-rightBorder+2;
+              rect2.top=content.top+topBorder-2;
+              rect2.bottom=content.bottom-bottomBorder+2;
+              
+              /* ignore */ DrawFocusRect(dc,&rect2);
             }
-
-            return;
+            
+            fillRect=false;
           }
           else {
-            rect.left=xOff;
-            rect.top=yOff;
-            rect.right=xOff+width;
-            rect.bottom=yOff+height;
-
             /* ignore */ DrawFrameControl(dc,
-                                          &rect,
+                                          &content,
                                           DFC_BUTTON,
                                           DFCS_BUTTONPUSH | (draw->selected ? DFCS_PUSHED : 0));
-            return;
+            fillRect=false;
           }
           break;
-                case Display::editComboBackgroundFillIndex:
+        case Display::editComboBackgroundFillIndex:
           if (HasTheming()) {
               return;
           }
           else {
-            rect.left=xOff;
-            rect.top=yOff;
-            rect.right=xOff+width;
-            rect.bottom=yOff+height;
-
-            /* ignore */ DrawEdge(dc,&rect,EDGE_SUNKEN,BF_RECT);
-                    return;
-                }
-                    break;
+            /* ignore */ DrawEdge(dc,&content,EDGE_SUNKEN,BF_RECT);
+            fillRect=false;
+          }
+          break;
         case Display::toolbarButtonBackgroundFillIndex:
           if (HasTheming()) {
-            RECT content,clip;
-
             if (draw->disabled) {
               style=TS_DISABLED;
             }
@@ -1548,178 +1534,95 @@ namespace Lum {
               style=TS_NORMAL;
             }
 
-            content.left=xOff;
-            content.right=xOff+width;
-            content.top=yOff;
-            content.bottom=yOff+height;
-
-            clip.left=x;
-            clip.right=x+w;
-            clip.top=y;
-            clip.bottom=y+h;
-
             if (MyIsThemeBackgroundPartiallyTransparent(toolbarTheme,TP_BUTTON,style)) {
               brush=::GetSysColorBrush(COLOR_BTNFACE);
-              rect.left=x;
-              rect.top=y;
-              rect.right=x+w;
-              rect.bottom=y+h;
 
-              /* ignore */ FillRect(dc,&rect,brush);
+              /* ignore */ FillRect(dc,&clip,brush);
             }
 
             MyDrawThemeBackground(toolbarTheme,dc,TP_BUTTON,style,&content,&clip);
 
-            return;
+            fillRect=false;
           }
           else {
-            rect.left=xOff;
-            rect.top=yOff;
-            rect.right=xOff+width;
-            rect.bottom=yOff+height;
-
             /* ignore */ DrawFrameControl(dc,
-                                          &rect,
+                                          &content,
                                           DFC_BUTTON,
                                           DFCS_BUTTONPUSH | (draw->selected ? DFCS_PUSHED : 0));
-            return;
+            fillRect=false;
           }
           break;
         case Display::entryBackgroundFillIndex:
           if (HasTheming()) {
-            RECT content,clip;
-
-            content.left=xOff;
-            content.right=xOff+width;
-            content.top=yOff;
-            content.bottom=yOff+height;
-
-            clip.left=x;
-            clip.right=x+w;
-            clip.top=y;
-            clip.bottom=y+h;
-
             /* ignore */ MyDrawThemeBackground(editTheme,dc,EP_EDITTEXT,ETS_NORMAL,&content,&clip);
           }
           else {
-            rect.left=xOff;
-            rect.top=yOff;
-            rect.right=xOff+width;
-            rect.bottom=yOff+height;
-            /* ignore */ DrawEdge(dc,&rect,EDGE_SUNKEN,BF_RECT);
+            /* ignore */ DrawEdge(dc,&content,EDGE_SUNKEN,BF_RECT);
           }
 
           if (draw->selected) {
-              if (draw->focused) {
-                brush=::GetSysColorBrush(COLOR_HIGHLIGHT);
-              }
-              else {
-              brush=::GetSysColorBrush(COLOR_BTNFACE);
-              }
+            if (draw->focused) {
+              brush=::GetSysColorBrush(COLOR_HIGHLIGHT);
             }
-          else {
-              brush=::GetSysColorBrush(COLOR_WINDOW);
+            else {
+            brush=::GetSysColorBrush(COLOR_BTNFACE);
             }
-
-            rect.left=xOff+leftBorder;
-            rect.top=yOff+topBorder;
-          rect.right=xOff+width-leftBorder-rightBorder+2;
-          rect.bottom=yOff+height-topBorder-bottomBorder+2;
-
-          draw->PushClip(x,y,w,h);
-          /* ignore */ FillRect(dc,&rect,brush);
-          draw->PopClip();
-
-          return;
-        case Display::hprogressBackgroundFillIndex:
-          if (HasTheming()) {
-            RECT content,clip;
-
-            content.left=xOff;
-            content.right=xOff+width;
-            content.top=yOff;
-            content.bottom=yOff+height;
-
-            clip.left=x;
-            clip.right=x+w;
-            clip.top=y;
-            clip.bottom=y+h;
-
-            MyDrawThemeBackground(progressTheme,dc,PP_BAR,1,&content,&clip);
-
-            return;
           }
           else {
-            /* ignore */ FillRect(dc,&rect,::GetSysColorBrush(COLOR_BTNFACE));
+            brush=::GetSysColorBrush(COLOR_WINDOW);
+          }
 
-            rect.left=xOff;
-            rect.top=yOff;
-            rect.right=xOff+width;
-            rect.bottom=yOff+height;
+          content.left+=leftBorder;
+          content.right-=rightBorder;
+          content.top+=topBorder;
+          content.bottom-=bottomBorder;
+          
+          /* ignore */ FillRect(dc,&content,brush);
 
-            /* ignore */ DrawEdge(dc,&rect,EDGE_SUNKEN,BF_RECT);
-            return;
+          fillRect=false;
+          break;
+        case Display::hprogressBackgroundFillIndex:
+          if (HasTheming()) {
+            MyDrawThemeBackground(progressTheme,dc,PP_BAR,1,&content,&clip);
+
+            fillRect=false;
+          }
+          else {
+            /* ignore */ FillRect(dc,&content,::GetSysColorBrush(COLOR_BTNFACE));
+            /* ignore */ DrawEdge(dc,&content,EDGE_SUNKEN,BF_RECT);
+            fillRect=false;
           }
           break;
         case Display::vprogressBackgroundFillIndex:
           if (HasTheming()) {
-            RECT content,clip;
-
-            content.left=xOff;
-            content.right=xOff+width;
-            content.top=yOff;
-            content.bottom=yOff+height;
-
-            clip.left=x;
-            clip.right=x+w;
-            clip.top=y;
-            clip.bottom=y+h;
-
             MyDrawThemeBackground(progressTheme,dc,PP_BARVERT,1,&content,&clip);
 
-            return;
+            fillRect=false;
           }
           else {
-            /* ignore */ FillRect(dc,&rect,::GetSysColorBrush(COLOR_BTNFACE));
-
-            rect.left=xOff;
-            rect.top=yOff;
-            rect.right=xOff+width;
-            rect.bottom=yOff+height;
-
-            /* ignore */ DrawEdge(dc,&rect,EDGE_SUNKEN,BF_RECT);
-            return;
+            /* ignore */ FillRect(dc,&content,::GetSysColorBrush(COLOR_BTNFACE));
+            /* ignore */ DrawEdge(dc,&content,EDGE_SUNKEN,BF_RECT);
+            fillRect=false;
           }
           break;
         case Display::hprogressFillIndex:
           if (HasTheming()) {
-            RECT content,extent,clip;
+            RECT extent;
             int  chunkSize;
             int  spaceSize;
 
             MyGetThemeInt(progressTheme,PP_CHUNK,1,TMT_PROGRESSCHUNKSIZE,&chunkSize);
             MyGetThemeInt(progressTheme,PP_CHUNK,1,TMT_PROGRESSSPACESIZE,&spaceSize);
 
-            content.left=xOff;
-            content.right=xOff+width;
-            content.top=yOff;
-            content.bottom=yOff+height;
-
             MyGetThemeBackgroundExtent(progressTheme,dc,PP_BAR,1,&content,&extent);
-
-            clip.left=x;
-            clip.right=x+w;
-            clip.top=y;
-            clip.bottom=y+h;
-
             MyDrawThemeBackground(progressTheme,dc,PP_BAR,1,&extent,&clip);
 
-            content.left=x;
+            content.left=0;
             content.right=content.left+chunkSize;
-            content.top=y;
-            content.bottom=y+h;
+            content.top=0;
+            content.bottom=height;
 
-            while (content.right<=x+w) {
+            while (content.right<=width) {
               MyDrawThemeBackground(progressTheme,dc,PP_CHUNK,1,&content,NULL);
               content.left+=(chunkSize+spaceSize);
               content.right=content.left+chunkSize;
@@ -1728,7 +1631,7 @@ namespace Lum {
             //content.right=x+w;
             //MyDrawThemeBackground(progressTheme,dc,PP_CHUNK,1,&content,NULL);
 
-            return;
+            fillRect=false;
           }
           else {
             brush=::GetSysColorBrush(COLOR_HIGHLIGHT);
@@ -1736,33 +1639,22 @@ namespace Lum {
           break;
         case Display::vprogressFillIndex:
           if (HasTheming()) {
-            RECT content,extent,clip;
+            RECT extent;
             int  chunkSize;
             int  spaceSize;
 
             MyGetThemeInt(progressTheme,PP_CHUNKVERT,1,TMT_PROGRESSCHUNKSIZE,&chunkSize);
             MyGetThemeInt(progressTheme,PP_CHUNKVERT,1,TMT_PROGRESSSPACESIZE,&spaceSize);
 
-            content.left=xOff;
-            content.right=xOff+width;
-            content.top=yOff;
-            content.bottom=yOff+height;
-
             MyGetThemeBackgroundExtent(progressTheme,dc,PP_BARVERT,1,&content,&extent);
-
-            clip.left=x;
-            clip.right=x+w;
-            clip.top=y;
-            clip.bottom=y+h;
-
             MyDrawThemeBackground(progressTheme,dc,PP_BARVERT,1,&extent,&clip);
 
-            content.left=x;
-            content.right=x+w;
-            content.bottom=y+h;
+            content.left=0;
+            content.right=width;
+            content.bottom=height;
             content.top=content.bottom-chunkSize;
 
-            while (content.top>=y) {
+            while (content.top>=0) {
               MyDrawThemeBackground(progressTheme,dc,PP_CHUNKVERT,1,&content,NULL);
               content.bottom-=(chunkSize+spaceSize);
               content.top=content.bottom-chunkSize;
@@ -1771,8 +1663,7 @@ namespace Lum {
             //content.top=y;
             //MyDrawThemeBackground(progressTheme,dc,PP_CHUNKVERT,1,&content,NULL);
 
-
-            return;
+            fillRect=false;
           }
           else {
             brush=::GetSysColorBrush(COLOR_HIGHLIGHT);
@@ -1781,26 +1672,14 @@ namespace Lum {
         case Display::hknobBoxFillIndex:
         case Display::vknobBoxFillIndex:
           if (HasTheming()) {
-            RECT extent,clip;
-
-            extent.left=xOff;
-            extent.right=xOff+width;
-            extent.top=yOff;
-            extent.bottom=yOff+height;
-
-            clip.left=x;
-            clip.right=x+w;
-            clip.top=y;
-            clip.bottom=y+h;
-
             if (fill==Display::hknobBoxFillIndex) {
-              MyDrawThemeBackground(scrollbarTheme,dc,SBP_LOWERTRACKHORZ,SCRBS_NORMAL,&extent,&clip);
+              MyDrawThemeBackground(scrollbarTheme,dc,SBP_LOWERTRACKHORZ,SCRBS_NORMAL,&content,&clip);
             }
             else {
-              MyDrawThemeBackground(scrollbarTheme,dc,SBP_LOWERTRACKVERT,SCRBS_NORMAL,&extent,&clip);
+              MyDrawThemeBackground(scrollbarTheme,dc,SBP_LOWERTRACKVERT,SCRBS_NORMAL,&content,&clip);
             }
 
-            return;
+            fillRect=false;
           }
           else {
             HBITMAP  bitmap;
@@ -1811,16 +1690,10 @@ namespace Lum {
 
             bitmap=::CreateBitmap(8,2,1,1,data);
             brush=::CreatePatternBrush(bitmap);
-
-            rect.left=x;
-            rect.top=y;
-            rect.right=x+w;
-            rect.bottom=y+h;
-
             oldTextColor=::SetTextColor(dc,GetSysColor(COLOR_WINDOW));
             oldBgColor=::SetBkColor(dc,GetSysColor(COLOR_SCROLLBAR));
 
-            /* ignore */ ::FillRect(dc,&rect,brush);
+            /* ignore */ ::FillRect(dc,&content,brush);
 
             /* ignore */ ::SetTextColor(dc,oldTextColor);
             /* ignore */ ::SetBkColor(dc,oldBgColor);
@@ -1828,30 +1701,18 @@ namespace Lum {
             ::DeleteObject(brush);
             ::DeleteObject(bitmap);
 
-            return;
+            fillRect=false;
           }
           break;
         case Display::tabBackgroundFillIndex:
           if (HasTheming()) {
-            FillRect(dc,&rect,::GetSysColorBrush(COLOR_INFOBK));
-
-            RECT content,extent,clip;
-
-            content.left=xOff;
-            content.right=xOff+width;
-            content.top=yOff;
-            content.bottom=yOff+height;
-
+            RECT extent;
+            
+            FillRect(dc,&content,::GetSysColorBrush(COLOR_INFOBK));
             MyGetThemeBackgroundExtent(tabTheme,dc,TABP_BODY,1,&content,&extent);
-
-            clip.left=x;
-            clip.right=x+w;
-            clip.top=y;
-            clip.bottom=y+h;
-
             MyDrawThemeBackground(tabTheme,dc,TABP_BODY,1,&extent,&clip);
 
-            return;
+            fillRect=false;
           }
           else {
             brush=::GetSysColorBrush(COLOR_BTNFACE);
@@ -1859,16 +1720,16 @@ namespace Lum {
           break;
         case Display::tabRiderBackgroundFillIndex:
           if (HasTheming()) {
-            FillRect(dc,&rect,::GetSysColorBrush(COLOR_INFOBK));
-
-            RECT content,extent,clip;
+            RECT tc,extent;
+            
+            FillRect(dc,&content,::GetSysColorBrush(COLOR_INFOBK));
 
             // Extent of tab frame
-            content.left=0;
-            content.right=0;
-            content.top=0;
-            content.bottom=0;
-            MyGetThemeBackgroundExtent(tabTheme,NULL,TABP_PANE,1,&content,&extent);
+            tc.left=0;
+            tc.right=0;
+            tc.top=0;
+            tc.bottom=0;
+            MyGetThemeBackgroundExtent(tabTheme,NULL,TABP_PANE,1,&tc,&extent);
 
             // Under the theming engine the tab rider contains the gap!
             // So draw the control larger by the amount of the gap
@@ -1882,26 +1743,17 @@ namespace Lum {
               style=TTIS_NORMAL;
             }
 
-            content.left=xOff;
-            content.right=xOff+width;
-            content.top=yOff;
-            content.bottom=yOff+height;
-
             MyGetThemeBackgroundExtent(tabTheme,dc,TABP_TOPTABITEM,style,&content,&extent);
-
-            clip.left=x;
-            clip.right=x+w;
-            clip.top=y;
-            clip.bottom=y+h;
-
             MyDrawThemeBackground(tabTheme,dc,TABP_TOPTABITEM,style,&extent,&clip);
 
-            return;
+            fillRect=false;
           }
           else {
-            FillRect(dc,&rect,::GetSysColorBrush(COLOR_BTNFACE));
+            FillRect(dc,&content,::GetSysColorBrush(COLOR_BTNFACE));
             // right
             draw->PushForeground(GetSysColor(COLOR_3DDKSHADOW));
+            
+            // TODO:
             draw->DrawLine(xOff+width-1,yOff+3,xOff+width-1,yOff+height-1);
             draw->DrawLine(xOff+width-2,yOff+1,xOff+width-2,yOff+height-1);
             draw->PopForeground();
@@ -1918,7 +1770,7 @@ namespace Lum {
 
             draw->PopForeground();
 
-            return;
+            fillRect=false;
           }
           break;
         case Display::columnBackgroundFillIndex:
@@ -1927,8 +1779,6 @@ namespace Lum {
         case Display::columnRightBackgroundFillIndex:
         case Display::columnEndBackgroundFillIndex:
           if (HasTheming()) {
-            RECT content,clip;
-
             if (draw->selected) {
               style=HIS_PRESSED;
             }
@@ -1939,52 +1789,25 @@ namespace Lum {
               style=HIS_NORMAL;
             }
 
-            content.left=xOff;
-            content.right=xOff+width;
-            content.top=yOff;
-            content.bottom=yOff+height;
-
-            clip.left=x;
-            clip.right=x+w;
-            clip.top=y;
-            clip.bottom=y+h;
-
             MyDrawThemeBackground(headerTheme,dc,HP_HEADERITEM,style,&content,&clip);
 
-            return;
+            fillRect=false;
           }
           else {
-            rect.left=xOff;
-            rect.top=yOff;
-            rect.right=xOff+width;
-            rect.bottom=yOff+height;
-
             /* ignore */ DrawFrameControl(dc,
-                                          &rect,
+                                          &content,
                                           DFC_BUTTON,
                                           DFCS_BUTTONPUSH | (draw->selected ? DFCS_PUSHED : 0));
-            return;
+            fillRect=false;
           }
           break;
         case Display::menuStripBackgroundFillIndex:
           if (HasTheming() && menuTheme!=NULL) {
-            RECT content,clip;
-
             style=1; // TODO
-
-            content.left=xOff;
-            content.right=xOff+width;
-            content.top=yOff;
-            content.bottom=yOff+height;
-
-            clip.left=x;
-            clip.right=x+w;
-            clip.top=y;
-            clip.bottom=y+h;
-
+            
             MyDrawThemeBackground(menuTheme,dc,MENU_BARBACKGROUND,style,&content,&clip);
 
-            return;
+            fillRect=false;
           }
           else {
             brush=::GetSysColorBrush(COLOR_BTNFACE);
@@ -1992,8 +1815,6 @@ namespace Lum {
           break;
         case Display::menuPulldownBackgroundFillIndex:
           if (HasTheming() && menuTheme!=NULL) {
-            RECT content,clip;
-
             if (draw->selected) {
               style=3; // TODO
             }
@@ -2004,24 +1825,16 @@ namespace Lum {
               style=1; // TODO
             }
 
-            content.left=xOff;
-            content.right=xOff+width;
-            content.top=yOff;
-            content.bottom=yOff+height;
-
-            clip.left=x;
-            clip.right=x+w;
-            clip.top=y;
-            clip.bottom=y+h;
-
             MyDrawThemeBackground(menuTheme,dc,MENU_BARBACKGROUND,style,&content,&clip);
             MyDrawThemeBackground(menuTheme,dc,MENU_BARITEM,style,&content,&clip);
 
-            return;
+            fillRect=false;
           }
           else {
-            /* ignore */ FillRect(dc,&rect,GetSysColorBrush(COLOR_BTNFACE));
+            /* ignore */ FillRect(dc,&content,GetSysColorBrush(COLOR_BTNFACE));
 
+            // TODO
+            
             // Outer area is always filled with background color
             draw->PushForeground(GetSysColor(COLOR_BTNFACE));
             draw->DrawRectangle(xOff,yOff,width,height);
@@ -2055,19 +1868,12 @@ namespace Lum {
               draw->PopForeground();
             }
 
-            return;
+            fillRect=false;
           }
           break;
         case Display::menuEntryBackgroundFillIndex:
           if (HasTheming() && menuTheme!=NULL) {
-            RECT content,clip;
-
-            rect.left=x;
-            rect.top=y;
-            rect.right=x+w;
-            rect.bottom=y+h;
-
-            /* ignore */ FillRect(dc,&rect,GetSysColorBrush(COLOR_MENU));
+            /* ignore */ FillRect(dc,&content,GetSysColorBrush(COLOR_MENU));
 
             if (draw->selected) {
               style=2; // TODO
@@ -2076,19 +1882,9 @@ namespace Lum {
               style=1; // TODO
             }
 
-            content.left=xOff;
-            content.right=xOff+width;
-            content.top=yOff;
-            content.bottom=yOff+height;
-
-            clip.left=x;
-            clip.right=x+w;
-            clip.top=y;
-            clip.bottom=y+h;
-
             MyDrawThemeBackground(menuTheme,dc,MENU_POPUPITEM,style,&content,&clip);
 
-            return;
+            fillRect=false;
           }
           else {
             if (draw->selected) {
@@ -2100,127 +1896,108 @@ namespace Lum {
           }
           break;
         case Display::graphBackgroundFillIndex:
-          /* ignore */ FillRect(dc,&rect,::GetSysColorBrush(COLOR_3DFACE));
-
-          rect.left=xOff;
-          rect.top=yOff;
-          rect.right=xOff+width;
-          rect.bottom=yOff+height;
-
-          /* ignore */ DrawEdge(dc,&rect,EDGE_SUNKEN,BF_RECT);
-          return;
+          /* ignore */ FillRect(dc,&content,::GetSysColorBrush(COLOR_3DFACE));
+          /* ignore */ DrawEdge(dc,&content,EDGE_SUNKEN,BF_RECT);
+          fillRect=false;
+          break;
         case Display::toolbarBackgroundFillIndex:
           brush=::GetSysColorBrush(COLOR_3DFACE);
           break;
         case Display::menuWindowBackgroundFillIndex:
 #if 0
           if (HasTheming()) {
-            /* ignore */ FrameRect(dc,&rect,GetSysColorBrush(COLOR_3DSHADOW));
+            /* ignore */ FrameRect(dc,&content,GetSysColorBrush(COLOR_3DSHADOW));
           }
           else {
 #endif
-          /* ignore */ FillRect(dc,&rect,::GetSysColorBrush(COLOR_3DFACE));
-
-          rect.left=xOff;
-          rect.top=yOff;
-          rect.right=xOff+width;
-          rect.bottom=yOff+height;
-
-          /* ignore */ DrawEdge(dc,&rect,EDGE_RAISED,BF_RECT);
+          /* ignore */ FillRect(dc,&content,::GetSysColorBrush(COLOR_3DFACE));
+          /* ignore */ DrawEdge(dc,&content,EDGE_RAISED,BF_RECT);
          //}
-          return;
+          fillRect=false;
+          break;
         case Display::popupWindowBackgroundFillIndex:
-          /* ignore */ FillRect(dc,&rect,::GetSysColorBrush(COLOR_3DFACE));
-
-          rect.left=xOff;
-          rect.top=yOff;
-          rect.right=xOff+width;
-          rect.bottom=yOff+height;
-
-          /* ignore */ DrawEdge(dc,&rect,EDGE_RAISED,BF_RECT);
-          return;
+          /* ignore */ FillRect(dc,&content,::GetSysColorBrush(COLOR_3DFACE));
+          /* ignore */ DrawEdge(dc,&content,EDGE_RAISED,BF_RECT);
+          fillRect=false;
+          break;
         case Display::dialogWindowBackgroundFillIndex:
-          /* ignore */ FillRect(dc,&rect,::GetSysColorBrush(COLOR_3DFACE));
-          return;
+          /* ignore */ FillRect(dc,&content,::GetSysColorBrush(COLOR_3DFACE));
+          fillRect=false;
+          break;
         case Display::tooltipWindowBackgroundFillIndex:
-          /* ignore */ FillRect(dc,&rect,::GetSysColorBrush(COLOR_INFOBK));
+          /* ignore */ FillRect(dc,&content,::GetSysColorBrush(COLOR_INFOBK));
+          
           draw->PushForeground(RGB(0,0,0));
-          draw->DrawRectangle(xOff,yOff,width,height);
+          draw->DrawRectangle(content.left,
+                              content.top,
+                              content.right-content.left,
+                              content.bottom-content.top);
           draw->PopForeground();
-          return;
+          fillRect=false;
+          break;
         case Display::listboxBackgroundFillIndex:
-          /* ignore */ FillRect(dc,&rect,::GetSysColorBrush(COLOR_WINDOW));
-          return;
+          /* ignore */ FillRect(dc,&content,::GetSysColorBrush(COLOR_WINDOW));
+          fillRect=false;
+          break;
         case Display::scrolledBackgroundFillIndex:
           if (HasTheming()) {
-            RECT content,clip;
-
-            content.left=xOff;
-            content.right=xOff+width;
-            content.top=yOff;
-            content.bottom=yOff+height;
-
-            clip.left=x;
-            clip.right=x+w;
-            clip.top=y;
-            clip.bottom=y+h;
-
             MyDrawThemeBackground(listboxTheme,dc,LBCP_BORDER_NOSCROLL,0,&content,&clip);
 
-            return;
+            fillRect=false;
           }
           else {
-            /* ignore */ FillRect(dc,&rect,::GetSysColorBrush(COLOR_3DFACE));
-
-            rect.left=xOff;
-            rect.top=yOff;
-            rect.right=xOff+width;
-            rect.bottom=yOff+height;
-
-            /* ignore */ DrawEdge(dc,&rect,EDGE_SUNKEN,BF_RECT);
-            return;
+            /* ignore */ FillRect(dc,&content,::GetSysColorBrush(COLOR_3DFACE));
+            /* ignore */ DrawEdge(dc,&content,EDGE_SUNKEN,BF_RECT);
+            fillRect=false;
           }
+          break;
         case Display::hscrollBackgroundFillIndex:
         case Display::vscrollBackgroundFillIndex:
-          return;
+          fillRect=false;
+          break;
         case Display::boxedBackgroundFillIndex:
-          /* ignore */ FillRect(dc,&rect,::GetSysColorBrush(COLOR_3DFACE));
-
-          rect.left=xOff;
-          rect.top=yOff;
-          rect.right=xOff+width;
-          rect.bottom=yOff+height;
-
-          /* ignore */ DrawEdge(dc,&rect,EDGE_SUNKEN,BF_RECT);
-          return;
+          /* ignore */ FillRect(dc,&content,::GetSysColorBrush(COLOR_3DFACE));
+          /* ignore */ DrawEdge(dc,&content,EDGE_SUNKEN,BF_RECT);
+          fillRect=false;
+          break;
         case Display::plateBackgroundFillIndex:
-          /* ignore */ FillRect(dc,&rect,::GetSysColorBrush(COLOR_3DFACE));
+          /* ignore */ FillRect(dc,&content,::GetSysColorBrush(COLOR_3DFACE));
           draw->PushForeground(RGB(0,0,0));
-          draw->DrawRectangle(xOff,yOff,width,height);
+          draw->DrawRectangle(content.left,
+                              content.top,
+                              content.right-content.left,
+                              content.bottom-content.top);
           draw->PopForeground();
-          return;
+          fillRect=false;
+          break;
         case Display::labelBackgroundFillIndex:
+            // TODO
             if (draw->focused) {
-              ///* ignore */ DrawFocusRect(dc,&rect);
-              draw->PushForeground(GetSysColor(COLOR_BTNTEXT));
-              draw->PushDash("\001\001",2,::Lum::OS::DrawInfo::fDash);
-              draw->DrawLine(xOff        ,yOff,         xOff+width-1,yOff);
-              draw->DrawLine(xOff+width-1,yOff,         xOff+width-1,yOff+height-1);
-              draw->DrawLine(xOff+width-1,yOff+height-1,xOff,        yOff+height-1);
-              draw->DrawLine(xOff        ,yOff+height-1,xOff,        yOff);
-              draw->PopDash();
-              draw->PopForeground();
+              /* ignore */ DrawFocusRect(dc,&content);
             }
-          return;
+          fillRect=false;
+          break;
         case Display::statusbarBackgroundFillIndex:
-          return;
+          fillRect=false;
+          break;
         default:
           std::cerr << "Missing background support for background " << fill << std::endl;
           assert(false);
           break;
         }
 
-        /* ignore */ FillRect(dc,&rect,brush);
+        if (fillRect) {
+          /* ignore */ FillRect(dc,&content,brush);
+        }
+        
+#if defined(LUM_HAVE_LIB_CAIRO)
+        if (dynamic_cast<Cairo::DrawInfo*>(draw)!=NULL) {
+          cairo_surface_mark_dirty(dynamic_cast<Cairo::DrawInfo*>(draw)->surface);
+        }
+#endif        
+        draw->CopyFromBitmap(bitmap,x-xOff,y-yOff,w,h,x,y);
+        
+        delete bitmap;
       }
 
       Frame::Frame(Display::FrameIndex frame)
@@ -2308,9 +2085,22 @@ namespace Lum {
 
       void Frame::Draw(Lum::OS::DrawInfo* draw, int x, int y, size_t w, size_t h)
       {
-        HDC  dc=dynamic_cast<DrawInfo*>(draw)->dc;
+        HDC  dc;
         RECT rect;
 
+        if (dynamic_cast<DrawInfo*>(draw)!=NULL) {
+          dc=dynamic_cast<DrawInfo*>(draw)->dc;
+        }
+#if defined(LUM_HAVE_LIB_CAIRO)
+        else if (dynamic_cast<Cairo::DrawInfo*>(draw)!=NULL) {
+          cairo_surface_flush(dynamic_cast<Cairo::DrawInfo*>(draw)->surface);
+          dc=cairo_win32_surface_get_dc(dynamic_cast<Cairo::DrawInfo*>(draw)->surface);
+        }
+#endif        
+        else {
+          assert(false);
+        }
+        
         if (minWidth==0 && minHeight==0) {
           return;
         }
@@ -2323,14 +2113,19 @@ namespace Lum {
         switch (frame) {
         case Display::tabFrameIndex:
           if (HasTheming()) {
+            HRGN region=CreateRectRgn(x,y,x+w,y+h);
+            HRGN tmpRegion=CreateRectRgn(x+leftBorder,y+topBorder,x+w-minWidth,y+h-minHeight);
 
-            draw->PushClipBegin(x,y,w,h);
-            draw->SubClipRegion(x+leftBorder,y+topBorder,w-minWidth,h-minHeight);
-            draw->PushClipEnd();
+            // Push clip for inner region
+            /* ignore */ CombineRgn(region,region,tmpRegion,RGN_DIFF);
+            /* ignore */ DeleteObject(tmpRegion);
+            /* ignore */ SelectClipRgn(dc,region);
 
             /* ignore */ MyDrawThemeBackground(tabTheme,dc,TABP_PANE,1,&rect,NULL);
 
-            draw->PopClip();
+            // Pop clip for inner region
+            /* ignore */ SelectClipRgn(dc,NULL);
+            /* ignore */ DeleteObject(region);
 
             RECT clip;
 
@@ -2345,7 +2140,6 @@ namespace Lum {
             clip.bottom=clip.top+topBorder;
 
             /* ignore */ MyDrawThemeBackground(tabTheme,dc,TABP_BODY,1,&rect,&clip);
-
           }
           else {
             draw->PushForeground(GetSysColor(COLOR_3DDKSHADOW));
@@ -2405,15 +2199,7 @@ namespace Lum {
           }
           break;
         case Display::focusFrameIndex:
-          ///* ignore */ DrawFocusRect(dc,&rect);
-          draw->PushForeground(GetSysColor(COLOR_BTNTEXT));
-          draw->PushDash("\001\001",2,::Lum::OS::DrawInfo::fDash);
-          draw->DrawLine(x          ,y,      x+w-1,y);
-          draw->DrawLine(x+w-1,y,            x+w-1,y+h-1);
-          draw->DrawLine(x+w-1,y+h-1,x,            y+h-1);
-          draw->DrawLine(x          ,y+h-1,x,      y);
-          draw->PopDash();
-          draw->PopForeground();
+          /* ignore */ DrawFocusRect(dc,&rect);
           break;
         case Display::statuscellFrameIndex:
           draw->PushForeground(GetSysColor(COLOR_3DSHADOW));
@@ -2481,6 +2267,12 @@ namespace Lum {
           assert(false);
           break;
         }
+        
+#if defined(LUM_HAVE_LIB_CAIRO)
+        if (dynamic_cast<Cairo::DrawInfo*>(draw)!=NULL) {
+          cairo_surface_mark_dirty(dynamic_cast<Cairo::DrawInfo*>(draw)->surface);
+        }
+#endif        
       }
 
       Win32Theme::Win32Theme(Lum::OS::Display *display)
