@@ -46,6 +46,14 @@
 #include <X11/Xatom.h>
 #include <X11/extensions/shape.h> // TODO: Add configure check
 
+#if defined(HAVE_LIB_XRANDR)
+  #include <X11/extensions/Xrandr.h>
+#endif
+
+#if defined(HAVE_LIB_GCONF)
+  #include <gconf/gconf-client.h>
+#endif
+
 #if defined(HAVE_LIB_GTK)
   #include <gtk/gtk.h>
   #include <gdk/gdkx.h>
@@ -210,6 +218,33 @@ namespace Lum {
         }
       };
 
+        // Find out dimensions of workSpace, this is size of desktop minus globals toolsbars etc...
+      void Display::GetWorkAreaDimension()
+      {
+        unsigned char* atomProperties=NULL;
+        Atom           atomType;
+        int            atomFormat;
+        unsigned long  atomItems;
+        unsigned long  atomBytes;
+        int            res;
+
+        res=XGetWindowProperty(display,RootWindow(display,scrNum),
+                               atoms[netWorkAreaAtom],2,2,false,atoms[cardinalAtom],
+                               &atomType,&atomFormat,&atomItems,&atomBytes,&atomProperties);
+
+        if (res==Success &&
+            atomProperties!=NULL) {
+          workAreaWidth=reinterpret_cast<uint32_t*>(atomProperties)[0];
+          workAreaHeight=reinterpret_cast<uint32_t*>(atomProperties)[1];
+        }
+        else {
+          workAreaWidth=0;
+          workAreaHeight=0;
+        }
+
+        XFree(atomProperties);
+      }
+
       double Display::GetDPI() const
       {
         return dpi;
@@ -318,24 +353,6 @@ namespace Lum {
 
 #endif
 
-      /**
-        Get the Illumination window matching the given X11 window.
-      */
-      Window* Display::GetWindow(::Window window)
-      {
-        std::list<Window*>::const_iterator iter;
-
-        iter=winList.begin();
-        while (iter!=winList.end()) {
-          if ((*iter)->window==window) {
-            return *iter;
-          }
-
-          ++iter;
-        }
-        return NULL;
-      }
-
       /**
         Return the Illumination tray belonging to the given X11 window.
       */
@@ -442,6 +459,22 @@ namespace Lum {
         winList.remove(w);
       }
 
+      /**
+        Get the Illumination window matching the given X11 window.
+      */
+      Window* Display::GetWindow(::Window window)
+      {
+        for (std::list<Window*>::const_iterator w=winList.begin();
+             w!=winList.end();
+             ++w) {
+          if ((*w)->window==window) {
+            return *w;
+          }
+        }
+
+        return NULL;
+      }
+
       /**
         Adds the tray to the internal list of trays.
       */
@@ -692,6 +725,7 @@ namespace Lum {
       }*/
 
 #if defined(HAVE_LIBSM)
+  #if 0
       static void callback_die(SmcConn smc_conn, SmPointer client_data)
       {
         ((Display*)OS::display)->CallbackDie(smc_conn,client_data);
@@ -721,6 +755,7 @@ namespace Lum {
       {
         ((Display*)OS::display)->CallbackICEIOError(connection);
       }
+  #endif
 #endif
 
 #if defined(HAVE_LIB_OSSO)
@@ -879,14 +914,8 @@ namespace Lum {
         const char*    result;
 #endif
         OS::FontRef    font;
-        Atom           atomType;
-        int            atomFormat;
-        unsigned long  atomItems;
-        unsigned long  atomBytes;
-        unsigned char* atomProperties;
-        int            res;
-
-        /*if (XInitThreads()==0) {
+        /*
+        if (XInitThreads()==0) {
           std::cerr << "Warning: X11 does not support multiple threads!" << std::endl;
         }*/
 
@@ -937,6 +966,18 @@ namespace Lum {
         colorMap=XDefaultColormap(display,scrNum);
 #endif
 
+#if defined(HAVE_LIB_XRANDR)
+        xrandrActive=false;
+        if (XRRQueryVersion(display,&xrandrMajor,&xrandrMinor)!=0) {
+          if (XRRQueryExtension(display,&xrandrEventBase,&xrandrErrorBase)) {
+            xrandrActive=true;
+            XRRSelectInput(display,
+                           XRootWindow(display,scrNum),
+                           RRScreenChangeNotifyMask);		
+          }
+        }
+#endif
+
 #if defined(HAVE_LIB_GTK)
         eventLoop=new GMainLoopEventLoop();
   #if defined(HAVE_LIB_GSTREAMER)
@@ -950,8 +991,8 @@ namespace Lum {
 
         Images::Factory::Set(driver->CreateImageFactory(this));
 
-        scrWidth=XDisplayWidth(display,scrNum);
-        scrHeight=XDisplayHeight(display,scrNum);
+        screenWidth=XDisplayWidth(display,scrNum);
+        screenHeight=XDisplayHeight(display,scrNum);
 
         type=typeGraphical;
 
@@ -1161,17 +1202,7 @@ namespace Lum {
 
       */
 
-        // Find out dimensions of workSpace, this is size of desktop minus globals toolsbars etc...
-        atomProperties=NULL;
-        res=XGetWindowProperty(display,RootWindow(display,scrNum),
-                               atoms[netWorkAreaAtom],2,2,false,atoms[cardinalAtom],
-                               &atomType,&atomFormat,&atomItems,&atomBytes,&atomProperties);
-        if (res==Success && atomProperties!=NULL) {
-          workAreaWidth=reinterpret_cast<uint32_t*>(atomProperties)[0];
-          workAreaHeight=reinterpret_cast<uint32_t*>(atomProperties)[1];
-        }
-        XFree(atomProperties);
-
+        GetWorkAreaDimension();
         /*
         d.contextTimer.SetSecs(D.prefs.contextTimer DIV 1000,D.prefs.contextTimer MOD 1000);
                       d.contextTimer.SetObject(d);
@@ -1696,8 +1727,7 @@ namespace Lum {
 
       void Display::GetEvent()
       {
-        Tray* currentTray;
-
+        Tray*  currentTray;
         XEvent e;
 
         XNextEvent(display,&e);
@@ -1705,6 +1735,13 @@ namespace Lum {
         if (XFilterEvent(&e,0)!=0) {
           return;
         }
+
+#if defined(HAVE_LIB_XRANDR)
+        if (xrandrActive) {
+          XRRUpdateConfiguration(&e);
+        }
+#endif
+
       /*
         IF d.contextTimer.active THEN
           IF (e.type=X11.ButtonPress) OR (e.type=X11.ButtonRelease) OR (e.type=X11.MotionNotify) OR
@@ -1714,6 +1751,37 @@ namespace Lum {
         END;
                               */
 
+#if defined(HAVE_LIB_XRANDR)
+        if (xrandrActive) {
+          if (e.type-xrandrEventBase==RRScreenChangeNotify) {
+            XRRScreenChangeNotifyEvent *xrandrEvent=(XRRScreenChangeNotifyEvent*)&e;
+            int                        width,height;
+
+            if (xrandrEvent->rotation==1 ||
+                xrandrEvent->rotation==4) {
+              width=xrandrEvent->width;
+              height=xrandrEvent->height;
+            }
+            else {
+              width=xrandrEvent->height;
+              height=xrandrEvent->width;
+            }
+
+            if (GetScreenWidth()!=(unsigned long)width ||
+                GetScreenHeight()!=(unsigned long)height) {
+              screenWidth=(unsigned long)width;
+              screenHeight=(unsigned long)height;
+
+              GetWorkAreaDimension();
+
+              TriggerScreenRotated();
+            }
+
+            return;
+          }
+        }
+#endif
+
         switch (e.type) {
         case SelectionClear:
           if (!selClearPend) {
@@ -1852,7 +1920,6 @@ namespace Lum {
           FD_SET(x11Fd,&set);
 
           if (now>=max) {
-            std::cout << "Timeout!" << std::endl;
             return false;
           }
           tmp=max;
@@ -1864,11 +1931,8 @@ namespace Lum {
           ret=select(x11Fd+1,&set,NULL,NULL,&timeout);
 
           if (ret>0) {
-            std::cout << "Wakeup!" << std::endl;
             if (XCheckTypedWindowEvent(display,window,PropertyNotify,&event)==True) {
-              std::cout << "Got event " << event.type << " " << event.xproperty.atom << std::endl;
               if (event.type==PropertyNotify && event.xproperty.atom==property) {
-                std::cout << "Match!" << std::endl;
                 return true;
               }
             }
