@@ -76,9 +76,10 @@ namespace Lum {
     Object::CalcSize();
   }
 
-  void PercentBar::Draw(int x, int y, size_t w, size_t h)
+  void PercentBar::Draw(OS::DrawInfo* draw,
+                        int x, int y, size_t w, size_t h)
   {
-    Object::Draw(x,y,w,h);
+    Object::Draw(draw,x,y,w,h);
 
     if (!OIntersect(x,y,w,h)) {
       return;
@@ -90,9 +91,8 @@ namespace Lum {
       return;
     }
 
-    double       sum;
-    OS::DrawInfo *draw=GetDrawInfo();
-    size_t       entries,count,current,prev;
+    double sum;
+    size_t entries,count,current,prev;
 
     sum=0;
     entries=0;
