@@ -97,6 +97,14 @@ namespace Lum {
     return reference;
   }
 
+  void Popup::ReinitWindow()
+  {
+    container=NULL;
+    main=NULL;
+
+    Window::ReinitWindow();
+  }
+
   void Popup::CalcSize()
   {
     assert(main!=NULL);
@@ -116,10 +124,11 @@ namespace Lum {
       }
       container->SetFlex(main->HorizontalFlex(),main->VerticalFlex()); // To resize to same size of reference
       container->SetMain(main,seamless);
-      container->SetWindow(window);
     }
 
+    container->SetWindow(window);
     container->CalcSize();
+    container->SetWindow(window);
 
     if (GetReference()!=NULL &&
         GetType()==OS::Window::typePopup) {
@@ -129,42 +138,44 @@ namespace Lum {
       }
     }
 
-    window->SetSize(container->GetOWidth(),container->GetOHeight());
-    window->SetMinWidth(window->GetWidth());
-    window->SetMinHeight(window->GetHeight());
-    window->SetMaxWidth(window->GetWidth());
-    window->SetMaxHeight(window->GetHeight());
+    if (!IsOpen()) {
+      window->SetSize(container->GetOWidth(),container->GetOHeight());
+      window->SetMinMaxSize(window->GetWidth(),
+                            window->GetHeight(),
+                            window->GetWidth(),
+                            window->GetHeight());
 
-    if (reference!=NULL) {
-      OS::Window* parent;
-      int         x,y;
+      if (reference!=NULL) {
+        OS::Window* parent;
+        int         x,y;
 
-      parent=reference->GetWindow();
-      x=parent->GetX()+reference->GetOX();
+        parent=reference->GetWindow();
+        x=parent->GetX()+reference->GetOX();
 
-      if (parent->GetY()+reference->GetOY()+reference->GetOHeight()+window->GetHeight()<OS::display->GetScreenHeight()) {
-        y=parent->GetY()+reference->GetOY()+reference->GetOHeight();
+        if (parent->GetY()+reference->GetOY()+reference->GetOHeight()+window->GetHeight()<OS::display->GetScreenHeight()) {
+          y=parent->GetY()+reference->GetOY()+reference->GetOHeight();
+        }
+        else {
+          y=parent->GetY()+reference->GetOY()-window->GetHeight();
+        }
+
+        x=Base::RoundRange(x,0,(int)(OS::display->GetScreenWidth()-1-window->GetWidth()));
+        y=Base::RoundRange(y,0,(int)(OS::display->GetScreenHeight()-1-window->GetHeight()));
+
+        window->SetPos(x,y);
+        SetPosition(OS::Window::positionManual,OS::Window::positionManual);
       }
       else {
-        y=parent->GetY()+reference->GetOY()-window->GetHeight();
-      }
-
-      x=Base::RoundRange(x,0,(int)(OS::display->GetScreenWidth()-1-window->GetWidth()));
-      y=Base::RoundRange(y,0,(int)(OS::display->GetScreenHeight()-1-window->GetHeight()));
-
-      window->SetPos(x,y);
-      SetPosition(OS::Window::positionManual,OS::Window::positionManual);
-    }
-    else {
-      if (window->GetX()+window->GetWidth()-1>OS::display->GetScreenWidth()
-          && window->GetWidth()<OS::display->GetScreenWidth()) {
-        window->SetPos(OS::display->GetScreenWidth()-window->GetWidth(),
-                       window->GetY());
-      }
-      if (window->GetY()+window->GetHeight()-1>OS::display->GetScreenHeight()
-          && window->GetHeight()<OS::display->GetScreenHeight()) {
-        window->SetPos(window->GetX(),
-                       OS::display->GetScreenHeight()-window->GetHeight());
+        if (window->GetX()+window->GetWidth()-1>OS::display->GetScreenWidth()
+            && window->GetWidth()<OS::display->GetScreenWidth()) {
+          window->SetPos(OS::display->GetScreenWidth()-window->GetWidth(),
+                         window->GetY());
+        }
+        if (window->GetY()+window->GetHeight()-1>OS::display->GetScreenHeight()
+            && window->GetHeight()<OS::display->GetScreenHeight()) {
+          window->SetPos(window->GetX(),
+                         OS::display->GetScreenHeight()-window->GetHeight());
+        }
       }
     }
 
