@@ -27,12 +27,10 @@
 namespace Lum {
 
   RGBControl::RGBControl()
-  : r(new Model::Int()),g(new Model::Int()),b(new Model::Int())
+  : r(new Model::ULong(0,255)),
+    g(new Model::ULong(0,255)),
+    b(new Model::ULong(0,255))
   {
-    r->SetRange(0,255);
-    g->SetRange(0,255);
-    b->SetRange(0,255);
-
     Observe(r);
     Observe(g);
     Observe(b);
@@ -89,19 +87,19 @@ namespace Lum {
     else if (model==r && this->model.Valid()) {
       OS::ColorValue color=this->model->Get();
 
-      color.SetRed(r->Get()/255.0);
+      color.SetRed(r->GetDouble()/255.0);
       this->model->Set(color);
     }
     else if (model==g && this->model.Valid()) {
       OS::ColorValue color=this->model->Get();
 
-      color.SetGreen(g->Get()/255.0);
+      color.SetGreen(g->GetDouble()/255.0);
       this->model->Set(color);
     }
     else if (model==b && this->model.Valid()) {
       OS::ColorValue color=this->model->Get();
 
-      color.SetBlue(b->Get()/255.0);
+      color.SetBlue(b->GetDouble()/255.0);
       this->model->Set(color);
     }
 
