@@ -26,8 +26,13 @@
 namespace Lum {
 
   Slider::Slider()
-  : boxFrame(NULL),knob(NULL),offset(0),corr(0),model(NULL),
-    selected(false),useScale(false)
+  : boxFrame(NULL),
+    knob(NULL),
+    offset(0),
+    corr(0),
+    model(NULL),
+    selected(false),
+    useScale(false)
   {
     if (!OS::display->GetTheme()->FocusOnEditableControlsOnly()) {
       SetCanFocus(true);
@@ -43,7 +48,7 @@ namespace Lum {
 
   bool Slider::SetModel(Base::Model* model)
   {
-    this->model=dynamic_cast<Model::Int*>(model);
+    this->model=dynamic_cast<Model::Number*>(model);
 
     Control::SetModel(this->model);
 
@@ -61,15 +66,11 @@ namespace Lum {
   }
   void Slider::Dec()
   {
-    if (model->Get()>model->GetMin()) {
-      model->Dec();
-    }
+    model->Dec();
   }
   void Slider::Inc()
   {
-    if (model->Get()<model->GetMax()) {
-      model->Inc();
-    }
+    model->Inc();
   }
 
   void Slider::Resync(Base::Model* model, const Base::ResyncMsg& msg)
@@ -123,11 +124,12 @@ namespace Lum {
 
     boxSize=width-boxFrame->minWidth-2*(knob->GetWidth()/2);
 
-    if (model->GetMin()==model->GetMax()) {
+    if (model->GetMinAsUnsignedLong()==model->GetMaxAsUnsignedLong()) {
       kStart=0;
     }
     else {
-      kStart=Base::RoundDiv((model->Get()-model->GetMin())*boxSize,model->GetMax()-model->GetMin());
+      kStart=Base::RoundDiv((model->GetUnsignedLong()-model->GetMinAsUnsignedLong())*boxSize,
+                            model->GetMaxAsUnsignedLong()-model->GetMinAsUnsignedLong());
     }
 
     knobX=x+boxFrame->leftBorder+kStart;
@@ -151,8 +153,6 @@ namespace Lum {
 
     draw->PopClip();
 
-    knob->Draw(draw,knobX,knobY);
-
     draw->PushClip(x+boxFrame->leftBorder+kStart+knobWidth,
                    y+offset,
                    width-boxFrame->minWidth-kStart-knobWidth,
@@ -166,6 +166,8 @@ namespace Lum {
 
     draw->PopClip();
 
+    knob->Draw(draw,knobX,knobY);
+
     draw->activated=false;
     draw->selected=false;
     draw->focused=false;
@@ -180,7 +182,7 @@ namespace Lum {
     if (useScale) {
       scale=new HScale();
       scale->SetParent(this);
-      scale->SetInterval(model->GetMin(),model->GetMax());
+      scale->SetInterval(model->GetMinAsUnsignedLong(),model->GetMaxAsUnsignedLong());
       scale->SetFlex(true,true);
       scale->CalcSize();
 
@@ -239,7 +241,9 @@ namespace Lum {
 
     boxFrame->Draw(draw,this->x,this->y+fOffset,this->width,fSize);
 
-    DrawKnob(draw);
+    if (model.Valid() && !model->IsNull()) {
+      DrawKnob(draw);
+    }
   }
 
   void HSlider::HandleMouseMovement(int x, int /*y*/)
@@ -250,18 +254,18 @@ namespace Lum {
       x=this->x+boxFrame->leftBorder+corr;
     }
 
-    newValue=model->GetMin()+
+    newValue=model->GetMinAsUnsignedLong()+
         ((x-(this->x+boxFrame->leftBorder)-corr)*
-            (model->GetMax()-model->GetMin()+1))/
+            (model->GetMaxAsUnsignedLong()-model->GetMinAsUnsignedLong()+1))/
         (width-boxFrame->minWidth-knob->GetWidth());
 
-    if (newValue>=model->GetMin() && newValue<=model->GetMax()) {
+    if (newValue>=model->GetMinAsUnsignedLong() && newValue<=model->GetMaxAsUnsignedLong()) {
       model->Set(newValue);
     }
-    else if (newValue<model->GetMin()) {
+    else if (newValue<model->GetMinAsUnsignedLong()) {
       model->SetToMin();
     }
-    else if (newValue>model->GetMax()) {
+    else if (newValue>model->GetMaxAsUnsignedLong()) {
       model->SetToMax();
     }
   }
@@ -321,10 +325,10 @@ namespace Lum {
 
   bool HSlider::HandleKeyEvent(const OS::KeyEvent& event)
   {
-    int old,tmp;
+    unsigned long old,tmp;
 
     if (event.type==OS::KeyEvent::down && model.Valid()) {
-      old=model->Get();
+      old=model->GetUnsignedLong();
       if (event.key==OS::keyLeft) {
         Dec();
       }
@@ -339,17 +343,17 @@ namespace Lum {
       }
       else if (event.key==OS::keyNext) {
         tmp=old;
-        tmp-=(model->GetMax()-model->GetMin()+1) / 10;
-        if (tmp<model->GetMin()) {
-          tmp=model->GetMin();
+        tmp-=(model->GetMaxAsUnsignedLong()-model->GetMinAsUnsignedLong()+1) / 10;
+        if (tmp<model->GetMinAsUnsignedLong()) {
+          tmp=model->GetMinAsUnsignedLong();
         }
         model->Set(tmp);
       }
       else if (event.key==OS::keyPrior) {
         tmp=old;
-        tmp+=(model->GetMax()-model->GetMin()+1) / 10;
-        if (tmp>model->GetMax()) {
-          tmp=model->GetMax();
+        tmp+=(model->GetMaxAsUnsignedLong()-model->GetMinAsUnsignedLong()+1) / 10;
+        if (tmp>model->GetMaxAsUnsignedLong()) {
+          tmp=model->GetMaxAsUnsignedLong();
         }
         model->Set(tmp);
       }
@@ -424,11 +428,12 @@ namespace Lum {
 
     boxSize=height-boxFrame->minHeight-knob->GetHeight();
 
-    if (model->GetMax()==model->GetMin()) {
+    if (model->GetMaxAsUnsignedLong()==model->GetMinAsUnsignedLong()) {
       kStart=0;
     }
     else {
-      kStart=boxSize-Base::RoundDiv((model->Get()-model->GetMin())*boxSize,model->GetMax()-model->GetMin());
+      kStart=boxSize-Base::RoundDiv((model->GetUnsignedLong()-model->GetMinAsUnsignedLong())*boxSize,
+                                    model->GetMaxAsUnsignedLong()-model->GetMinAsUnsignedLong());
     }
 
     knobX=x+offset;
@@ -452,8 +457,6 @@ namespace Lum {
 
     draw->PopClip();
 
-    knob->Draw(draw,knobX,knobY);
-
     draw->PushClip(x+offset,
                    y+boxFrame->topBorder+kStart+knobHeight,
                    knobWidth,
@@ -467,6 +470,8 @@ namespace Lum {
 
     draw->PopClip();
 
+    knob->Draw(draw,knobX,knobY);
+
     draw->activated=false;
     draw->focused=false;
     draw->selected=false;
@@ -481,7 +486,7 @@ namespace Lum {
     if (useScale) {
       scale=new VScale();
       scale->SetParent(this);
-      scale->SetInterval(model->GetMin(),model->GetMax());
+      scale->SetInterval(model->GetMinAsUnsignedLong(),model->GetMaxAsUnsignedLong());
       scale->SetFlex(true,true);
       scale->CalcSize();
 
@@ -539,7 +544,9 @@ namespace Lum {
 
     draw->focused=false;
 
-    DrawKnob(draw);
+    if (model.Valid() && !model->IsNull()) {
+      DrawKnob(draw);
+    }
   }
 
   void VSlider::HandleMouseMovement(int /*x*/, int y)
@@ -550,15 +557,15 @@ namespace Lum {
       y=this->y+boxFrame->topBorder+corr;
     }
 
-    newValue=model->GetMax()-((y-(this->y+boxFrame->topBorder)-corr)*(model->GetMax()-model->GetMin()+1))/(height-boxFrame->minHeight-knob->GetHeight());
+    newValue=model->GetMaxAsUnsignedLong()-((y-(this->y+boxFrame->topBorder)-corr)*(model->GetMaxAsUnsignedLong()-model->GetMinAsUnsignedLong()+1))/(height-boxFrame->minHeight-knob->GetHeight());
 
-    if (newValue>=model->GetMin() && newValue<=model->GetMax()) {
+    if (newValue>=model->GetMinAsUnsignedLong() && newValue<=model->GetMaxAsUnsignedLong()) {
       model->Set(newValue);
     }
-    else if (newValue<model->GetMin()) {
+    else if (newValue<model->GetMinAsUnsignedLong()) {
       model->SetToMin();
     }
-    else if (newValue>model->GetMax()) {
+    else if (newValue>model->GetMaxAsUnsignedLong()) {
       model->SetToMax();
     }
   }
@@ -618,10 +625,10 @@ namespace Lum {
 
   bool VSlider::HandleKeyEvent(const OS::KeyEvent& event)
   {
-    int old,tmp;
+    unsigned long old,tmp;
 
     if (event.type==OS::KeyEvent::down && model.Valid()) {
-      old=model->Get();
+      old=model->GetUnsignedLong();
       if (event.key==OS::keyDown) {
         Dec();
       }
@@ -636,17 +643,17 @@ namespace Lum {
       }
       else if (event.key==OS::keyNext) {
         tmp=old;
-        tmp-=(model->GetMax()-model->GetMin()+1) / 10;
-        if (tmp<model->GetMin()) {
-          tmp=model->GetMin();
+        tmp-=(model->GetMaxAsUnsignedLong()-model->GetMinAsUnsignedLong()+1) / 10;
+        if (tmp<model->GetMinAsUnsignedLong()) {
+          tmp=model->GetMinAsUnsignedLong();
         }
         model->Set(tmp);
       }
       else if (event.key==OS::keyPrior) {
         tmp=old;
-        tmp+=(model->GetMax()-model->GetMin()+1) / 10;
-        if (tmp>model->GetMax()) {
-          tmp=model->GetMax();
+        tmp+=(model->GetMaxAsUnsignedLong()-model->GetMinAsUnsignedLong()+1) / 10;
+        if (tmp>model->GetMaxAsUnsignedLong()) {
+          tmp=model->GetMaxAsUnsignedLong();
         }
         model->Set(tmp);
       }
