@@ -22,7 +22,7 @@
 namespace Lum {
 
   State::State(bool active)
-  : boolModel(NULL),sizeTModel(NULL),index(1),active(active)
+  : boolModel(NULL),numberModel(NULL),index(1),active(active)
   {
     SetCanFocus(active);
     if (active && !OS::display->GetTheme()->FocusOnEditableControlsOnly()) {
@@ -53,13 +53,13 @@ namespace Lum {
   bool State::SetModel(Base::Model* model)
   {
     boolModel=dynamic_cast<Model::Boolean*>(model);
-    sizeTModel=dynamic_cast<Model::SizeT*>(model);
+    numberModel=dynamic_cast<Model::Number*>(model);
 
     if (boolModel.Valid()) {
       Control::SetModel(boolModel);
     }
     else {
-      Control::SetModel(sizeTModel);
+      Control::SetModel(numberModel);
     }
 
     return (GetModel()!=NULL);
@@ -68,7 +68,7 @@ namespace Lum {
   bool State::HandleMouseEvent(const OS::MouseEvent& event)
   {
     if (!active || !visible || GetModel()==NULL || !GetModel()->IsEnabled() ||
-        (sizeTModel.Valid() && sizeTModel->IsNull()) ||
+        (numberModel.Valid() && numberModel->IsNull()) ||
         (boolModel.Valid() && boolModel->IsNull())) {
       return false;
     }
@@ -80,8 +80,8 @@ namespace Lum {
         boolModel->Set(!orgState);
       }
       else {
-        orgState=sizeTModel->Get();
-        sizeTModel->Set(index);
+        orgState=numberModel->GetUnsignedLong();
+        numberModel->Set(index);
       }
 
       return true;
@@ -92,7 +92,7 @@ namespace Lum {
           boolModel->Set(orgState==0);
         }
         else {
-          sizeTModel->Set(index);
+          numberModel->Set(index);
         }
       }
       else {
@@ -100,7 +100,7 @@ namespace Lum {
           boolModel->Set(orgState!=0);
         }
         else {
-          sizeTModel->Set(orgState);
+          numberModel->Set(orgState);
         }
       }
       return true;
@@ -117,7 +117,7 @@ namespace Lum {
     }
 
     if (!active || !visible || GetModel()==NULL || !GetModel()->IsEnabled() ||
-        (sizeTModel.Valid() && sizeTModel->IsNull()) ||
+        (numberModel.Valid() && numberModel->IsNull()) ||
         (boolModel.Valid() && boolModel->IsNull())) {
       return false;
     }
@@ -127,8 +127,8 @@ namespace Lum {
         boolModel->Toggle();
         return true;
       }
-      else if (sizeTModel.Valid()) {
-        sizeTModel->Set(index);
+      else if (numberModel.Valid()) {
+        numberModel->Set(index);
         return true;
       }
     }
@@ -138,6 +138,8 @@ namespace Lum {
 
   void State::PrepareForBackground(OS::DrawInfo* draw)
   {
+    draw->selected=false;
+
     if (boolModel.Valid() &&
         !boolModel->IsNull() &&
         boolModel->IsEnabled()) {
@@ -145,10 +147,10 @@ namespace Lum {
         draw->selected=true;
       }
     }
-    else if (sizeTModel.Valid() &&
-             !sizeTModel->IsNull() &&
-             sizeTModel->IsEnabled()) {
-      if (sizeTModel->Get()==index) {
+    else if (numberModel.Valid() &&
+             !numberModel->IsNull() &&
+             numberModel->IsEnabled()) {
+      if (numberModel->GetUnsignedLong()==index) {
         draw->selected=true;
       }
     }
@@ -171,6 +173,8 @@ namespace Lum {
 
     /* --- */
 
+    draw->selected=false;
+
     if (boolModel.Valid() &&
         !boolModel->IsNull() &&
         boolModel->IsEnabled()) {
@@ -178,10 +182,10 @@ namespace Lum {
         draw->selected=true;
       }
     }
-    else if (sizeTModel.Valid() &&
-             !sizeTModel->IsNull() &&
-             sizeTModel->IsEnabled()) {
-      if (sizeTModel->Get()==index) {
+    else if (numberModel.Valid() &&
+             !numberModel->IsNull() &&
+             numberModel->IsEnabled()) {
+      if (numberModel->GetUnsignedLong()==index) {
         draw->selected=true;
       }
     }
@@ -202,7 +206,7 @@ namespace Lum {
 
   void State::Resync(Base::Model* model, const Base::ResyncMsg& msg)
   {
-    if (model==boolModel || model==sizeTModel) {
+    if (model==boolModel || model==numberModel) {
       Redraw();
     }
 
