@@ -31,13 +31,17 @@ namespace Lum {
   class TabButton : public Control
   {
   private:
-    Object          *image;      //! The label of the button
-    Model::SizeTRef model;
-    size_t          index;
-    size_t          orgIndex;
+    Tab              *tab;        //! The parent tab control
+    Object           *image;      //! The label of the button
+    Model::NumberRef model;       //! The model to decided if we are selected or not
+    size_t           index;       //! Our index value
+    size_t           orgIndex;    //! The original index value before we got selected
 
   public:
-    TabButton(size_t index, Object* object, const Model::SizeTRef& model);
+    TabButton(Tab* tab,
+              size_t index,
+              Object* object,
+              const Model::NumberRef& model);
     ~TabButton();
 
     bool HandlesKeyFocus() const;
@@ -57,9 +61,16 @@ namespace Lum {
     void Resync(Base::Model* model, const Base::ResyncMsg& msg);
   };
 
-  TabButton::TabButton(size_t index, Object* object, const Model::SizeTRef& model)
-  : image(object),model(model),index(index)
+  TabButton::TabButton(Tab* tab,
+                       size_t index,
+                       Object* object,
+                       const Model::NumberRef& model)
+  : tab(tab),
+    image(object),
+    model(model),
+    index(index)
   {
+    assert(tab!=NULL);
     assert(image!=NULL);
     assert(model.Valid());
 
@@ -89,7 +100,7 @@ namespace Lum {
            model.Valid() &&
            model->IsEnabled() &&
            !model->IsNull() &&
-           model->Get()==index;
+           model->GetUnsignedLong()==index;
   }
 
   void TabButton::CalcSize()
@@ -129,7 +140,7 @@ namespace Lum {
     if (event.type==OS::MouseEvent::down &&
         PointIsIn(event) &&
         event.button==OS::MouseEvent::button1) {
-      orgIndex=model->Get();
+      orgIndex=model->GetUnsignedLong();
       model->Set(index);
       return true;
     }
@@ -168,10 +179,10 @@ namespace Lum {
         return true;
       }
       else if (event.key==OS::keyLeft) {
-        dynamic_cast<Tab*>(parent)->ActivatePreviousTab();
+        tab->ActivatePreviousTab();
       }
       else if (event.key==OS::keyRight) {
-        dynamic_cast<Tab*>(parent)->ActivateNextTab();
+        tab->ActivateNextTab();
       }
     }
 
@@ -208,7 +219,7 @@ namespace Lum {
   void TabButton::PrepareForBackground(OS::DrawInfo* draw)
   {
     draw->activated=IsMouseActive();
-    draw->selected=model.Valid() && !model->IsNull() && model->Get()==index;
+    draw->selected=model.Valid() && !model->IsNull() && model->GetUnsignedLong()==index;
     draw->focused=HasFocus();
   }
 
@@ -224,7 +235,7 @@ namespace Lum {
     /* --- */
 
     draw->activated=IsMouseActive();
-    draw->selected=model.Valid() && !model->IsNull() && model->Get()==index;
+    draw->selected=model.Valid() && !model->IsNull() && model->GetUnsignedLong()==index;
     draw->focused=HasFocus();
 
     image->Draw(draw,x,y,w,h);
@@ -257,11 +268,15 @@ namespace Lum {
   /* ----------------------- */
 
   Tab::Tab()
-  : mw(0),mh(0),lastDrawn(NULL),frameGroup(new FrameGroup()),multi(new Multi())
+  : mw(0),
+    mh(0),
+    lastDrawn(NULL),
+    frameGroup(new FrameGroup()),
+    multi(new Multi())
   {
     SetCanFocus(true);
 
-    SetModel(new Model::SizeT(0));
+    SetModel(new Model::Number(0ul));
 
     frameGroup->SetFrame(new OS::EmptyFrame());
     frameGroup->SetBackground(OS::display->GetFill(OS::Display::tabBackgroundFillIndex));
@@ -289,16 +304,16 @@ namespace Lum {
 
   bool Tab::VisitChildren(Visitor &visitor, bool onlyVisible)
   {
+    if (!visitor.Visit(frameGroup)) {
+      return false;
+    }
+
     for (std::list<Object*>::iterator iter=list.begin(); iter!=list.end(); ++iter) {
       if (!visitor.Visit(*iter)) {
         return false;
       }
     }
 
-    if (!visitor.Visit(frameGroup)) {
-      return false;
-    }
-
     return true;
   }
 
@@ -309,7 +324,7 @@ namespace Lum {
 
     Object *rider;
 
-    rider=new TabButton(list.size(),label,model);
+    rider=new TabButton(this,list.size(),label,model);
     rider->SetParent(this);
     rider->SetFlex(true,true);
 
@@ -331,7 +346,7 @@ namespace Lum {
       return;
     }
 
-    if (model->Get()==0) {
+    if (model->GetUnsignedLong()==0) {
       model->Set(list.size()-1);
     }
     else {
@@ -345,7 +360,7 @@ namespace Lum {
       return;
     }
 
-    if (model->Get()==list.size()-1) {
+    if (model->GetUnsignedLong()==list.size()-1) {
       model->Set(0);
     }
     else {
@@ -355,7 +370,7 @@ namespace Lum {
 
   bool Tab::SetModel(Base::Model* model)
   {
-    this->model=dynamic_cast<Model::SizeT*>(model);
+    this->model=dynamic_cast<Model::Number*>(model);
 
     Control::SetModel(this->model);
 
@@ -437,7 +452,6 @@ namespace Lum {
                  int x, int y, size_t w, size_t h)
   {
     if (lastDrawn!=NULL) {
-
      /*
         This is a little trick:
         We must hide the old object, but we do not want it to clean up the
@@ -466,7 +480,7 @@ namespace Lum {
     OS::FrameRef tabFrame=OS::display->GetFrame(OS::Display::tabFrameIndex);
 
     // Draw frame around object area
-    tabFrame->SetGap(OS::display->GetTheme()->GetFirstTabOffset()+model->Get()*mw,mw,0);
+    tabFrame->SetGap(OS::display->GetTheme()->GetFirstTabOffset()+model->GetUnsignedLong()*mw,mw,0);
 
     tabFrame->Draw(draw,
                    this->x,
@@ -479,7 +493,7 @@ namespace Lum {
   {
     if (model==this->model &&
         visible) {
-      size_t                       pos=this->model->Get();
+      unsigned long                pos=this->model->GetUnsignedLong();
       std::list<Object*>::iterator iter=list.begin();
       Dialog                       *dialog=dynamic_cast<Dialog*>(GetWindow()->GetMaster());
 
