@@ -31,12 +31,10 @@ namespace Lum {
 
   Time::Time()
   : format(Base::timeFormatHM),
-    hour(new Model::Int()),minute(new Model::Int()),second(new Model::Int())
+    hour(new Model::ULong(0,23)),
+    minute(new Model::ULong(0,59)),
+    second(new Model::ULong(0,59))
   {
-    hour->SetRange(0,23);
-    minute->SetRange(0,59);
-    second->SetRange(0,59);
-
     Observe(hour);
     Observe(minute);
     Observe(second);
@@ -113,19 +111,19 @@ namespace Lum {
     else if (model==hour && this->model.Valid()) {
       Base::Time time=this->model->Get();
 
-      time.SetHour((unsigned char)hour->Get());
+      time.SetHour((unsigned char)hour->GetUnsignedLong());
       this->model->Set(time);
     }
     else if (model==minute && this->model.Valid()) {
       Base::Time time=this->model->Get();
 
-      time.SetMinute((unsigned char)minute->Get());
+      time.SetMinute((unsigned char)minute->GetUnsignedLong());
       this->model->Set(time);
     }
     else if (model==second && this->model.Valid()) {
       Base::Time time=this->model->Get();
 
-      time.SetSecond((unsigned char)second->Get());
+      time.SetSecond((unsigned char)second->GetUnsignedLong());
       this->model->Set(time);
     }
 
