@@ -115,6 +115,34 @@ namespace Lum {
 
     return t;
   }
+
+  Toggle* Toggle::Create(Object* label,
+                         Base::Model* model,
+                         bool horizontalFlex, bool verticalFlex)
+  {
+    Toggle *t=new Toggle();
+
+    t->SetFlex(horizontalFlex,verticalFlex);
+    t->SetModel(model);
+    t->SetLabel(label);
+
+    return t;
+  }
+
+  Toggle* Toggle::Create(const std::wstring& label,
+                         Base::Model* model,
+                         unsigned long index,
+                         bool horizontalFlex, bool verticalFlex)
+  {
+    Toggle *t=new Toggle();
+
+    t->SetFlex(horizontalFlex,verticalFlex);
+    t->SetModel(model);
+    t->SetIndex(index);
+    t->SetLabel(label);
+
+    return t;
+  }
 }
 
 
