@@ -55,6 +55,7 @@ namespace Lum {
     if (visible) {
       if (this->men!=NULL) {
         this->men->CalcSize();
+        this->men->SetWindow(GetWindow());    
         this->men->ResizeWidth(width);
       }
 
@@ -107,19 +108,23 @@ namespace Lum {
 
   bool WindowGroup::VisitChildren(Visitor &visitor, bool /*onlyVisible*/)
   {
-    if (men!=NULL && !visitor.Visit(men)) {
+    if (men!=NULL &&
+        !visitor.Visit(men)) {
       return false;
     }
 
-    if (main!=NULL && !visitor.Visit(main)) {
+    if (main!=NULL &&
+        !visitor.Visit(main)) {
       return false;
     }
 
-    if (tool!=NULL && !visitor.Visit(tool)) {
+    if (tool!=NULL &&
+        !visitor.Visit(tool)) {
       return false;
     }
 
-    if (status!=NULL && !visitor.Visit(status)) {
+    if (status!=NULL &&
+        !visitor.Visit(status)) {
       return false;
     }
 
